/** 
 *******************************************************************************
 * @file    txz_adc_ex.h
 * @brief   This file provides all the functions prototypes for ADC driver.
 * @brief   Extension Class.
 * @version V1.0.0.0
 * $Date:: 2017-10-30 17:48:20 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __ADC_EX_H
#define __ADC_EX_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_adc.h"

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @defgroup ADC ADC
 *  @brief    ADC Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Exported_define ADC Exported Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ADC_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Exported_define ADC Exported Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ADC_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Exported_typedef ADC Exported Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Channel Setting. \n
*/
/*----------------------------------*/
typedef struct
{
    uint32_t interrupt; /*!< Interrupt Enable/Disable.
                                : Use @ref ADC_IntEnable.       */
    uint32_t type;      /*!< Conversion Type.
                                : Use @ref ADC_Conversion.      */
    uint32_t ain;       /*!< AIN.
                                : Range is @ref ADC_AINRange.   */
} adc_ex_channel_setting_t;

/*----------------------------------*/
/** 
 * @brief  ADC handle structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    adc_t     super;            /*!< Driver internal information.      */
    struct
    {
        void (*single)(uint32_t id, TXZ_Result result);        /*!< Notity Single Conversion Done.       */
        void (*continuity)(uint32_t id, TXZ_Result result);    /*!< Notity Continuity Conversion Done.   */
        void (*trigger)(uint32_t id, TXZ_Result result);       /*!< Notity Trigger Conversion Done.      */
    } handler;
} adc_ex_t;

/**
 *  @}
 */ /* End of group ADC_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Exported_functions ADC Exported Functions
 *  @{
 */
TXZ_Result adc_ex_init(adc_ex_t *p_obj);
TXZ_Result adc_ex_deinit(adc_ex_t *p_obj);
TXZ_Result adc_ex_channel_setting(adc_ex_t *p_obj, uint32_t ch, adc_ex_channel_setting_t *p_setting);
TXZ_Result adc_ex_channel_clear(adc_ex_t *p_obj, uint32_t ch);
TXZ_Result adc_ex_channel_get_value(adc_ex_t *p_obj, uint32_t ch, uint32_t *p_value);
TXZ_Result adc_ex_start(adc_ex_t *p_obj);
TXZ_Result adc_ex_stop(adc_ex_t *p_obj);
TXZ_Result adc_ex_get_status(adc_ex_t *p_obj, uint32_t *p_status);
void adc_ex_single_irq_handler(adc_ex_t *p_obj);
void adc_ex_continuity_irq_handler(adc_ex_t *p_obj);
void adc_ex_trigger_irq_handler(adc_ex_t *p_obj);
/**
 *  @}
 */ /* End of group ADC_Exported_functions */

/**
 *  @}
 */ /* End of group ADC */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __ADC_EX_H */


