/** 
 *******************************************************************************
 * @file    txz_adc_sh.h
 * @brief   This file provides all the functions prototypes for ADC driver. \n
 *          Channel Class.
 * @version V1.0.0.0
 * $Date:: 2017-10-30 17:48:20 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __ADC_SH_H
#define __ADC_SH_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @defgroup ADC ADC
 *  @brief    ADC Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Exported_define ADC Exported Define
 *  @{
 */
 
/* no define */

/**
 *  @}
 */ /* End of group ADC_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Exported_define ADC Exported Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ADC_SamplingHold_Select */



/**
 *  @}
 */ /* End of group ADC_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Exported_typedef ADC Exported Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Initialize Setting.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t trgpat1;   /*!< Trigger Combnation Upper pattern select .
                                : Use @ref ADC_SH_TriggerPattern.    */
    uint32_t trgpat2;   /*!< Trigger Combnation Lower pattern select
                                : Use @ref ADC_SH_TriggerPattern.    */
} adc_sh_setting_t;

/*----------------------------------*/
/** 
 * @brief  ADC SH handle structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    TSB_SH_TypeDef       *p_instance;     /*!< Registers base address.           */
    adc_sh_setting_t     init;            /*!< Initial setting.                  */
} adc_sh_t;

/**
 *  @}
 */ /* End of group ADC_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Exported_functions ADC Exported Functions
 *  @{
 */
TXZ_Result adc_sh_init(adc_sh_t *p_obj);
TXZ_Result adc_sh_deinit(adc_sh_t *p_obj);
TXZ_Result adc_sh_set(adc_sh_t *p_obj, uint32_t upper_pattern, uint32_t lower_pattern);
TXZ_Result adc_sh_get(adc_sh_t *p_obj, uint32_t *p_value);
/**
 *  @}
 */ /* End of group ADC_Exported_functions */

/**
 *  @}
 */ /* End of group ADC */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __ADC_SH_H */


