/** 
 *******************************************************************************
 * @file    txz_gpio.h
 * @brief   This file provides all the functions prototypes for GPIO driver.
 * @version V1.0.0.0
 * $Date:: 2017-10-30 17:49:20 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __GPIO_H
#define __GPIO_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_driver_def.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @defgroup GPIO GPIO
 *  @brief    GPIO Driver.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup GPIO_Exported_define GPIO Exported Define
 *  @{
 */


/** 
 *  @defgroup GPIO_Result  Result
 *  @brief    GPIO Result Macro Definition.
 *  @{
 */
#define GPIO_RESULT_SUCCESS     (0)          /*!< Success       */
#define GPIO_RESULT_FAILURE     (-1)         /*!< Failure       */
#define GPIO_READ_FAILURE       (0xFFFFFFFF) /*!< Failure       */
/**
 *  @}
 */ /* End of group GPIO_Result */

/**
 *  @}
 */ /* End of group GPIO_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** @defgroup GPIO_Exported_Typedef GPIO Exported Typedef
  * @{
  */

/** 
 *  @enum gpio_pinstate_t
 *  @brief    Pin State Reset/Set Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PIN_RESET = 0,  /*!< 0: Clear       */
  GPIO_PIN_SET,        /*!< 1: Set         */
}gpio_pinstate_t;

/**
 *  @enum gpio_pininout_t
 *  @brief    Pin Input/Output Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PIN_INPUT = 0,  /*!< 0: Input       */
  GPIO_PIN_OUTPUT,     /*!< 1: Output       */
  GPIO_PIN_INOUT,      /*!< 2: Input/Output */
}gpio_pininout_t;
 
/** 
 *  @enum gpio_gr_t
 *  @brief    Port Group Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PORT_A = 0x0, /*!< 0: PA       */
  GPIO_PORT_B,       /*!< 1: PB       */
  GPIO_PORT_C,       /*!< 2: PC       */
  GPIO_PORT_D,       /*!< 3: PD       */
  GPIO_PORT_E,       /*!< 4: PE       */
  GPIO_PORT_F,       /*!< 5: PF       */
  GPIO_PORT_G,       /*!< 6: PG       */
  GPIO_PORT_H,       /*!< 7: PH       */
  GPIO_PORT_J,       /*!< 8: PJ       */
  GPIO_PORT_K,       /*!< 9: PK       */
  GPIO_PORT_L,       /*!< 10:PL       */
  GPIO_GROUP_Max     /*!< Max Number   */
}gpio_gr_t;

/** 
 *  @enum gpio_num_t
 *  @brief    Port Number Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PORT_0 = 0x0,  /*!< 0: Port0       */
  GPIO_PORT_1 = 0x1,  /*!< 1: Port1       */
  GPIO_PORT_2 = 0x2,  /*!< 2: Port2       */
  GPIO_PORT_3 = 0x3,  /*!< 3: Port3       */
  GPIO_PORT_4 = 0x4,  /*!< 4: Port4       */
  GPIO_PORT_5 = 0x5,  /*!< 5: Port5       */
  GPIO_PORT_6 = 0x6,  /*!< 6: Port6       */
  GPIO_PORT_7 = 0x7,  /*!< 7: Port7       */
  GPIO_PORT_Max = 0x08  /*!< Max Number     */
}gpio_num_t;

/** 
 *  @enum gpio_mode_t
 *  @brief    Port Mode Enumerated Type Definition.
 */
typedef enum
{
  GPIO_Mode_DATA = 0x0,  /*!< 0x0: PxDATA       */
  GPIO_Mode_CR  = 0x4,   /*!< 0x4: PxCR         */
  GPIO_Mode_FR1 = 0x8,   /*!< 0x8: PxFR1        */
  GPIO_Mode_FR2 = 0xC,   /*!< 0xC: PxFR2        */
  GPIO_Mode_FR3 = 0x10,  /*!< 0x10: PxFR3        */
  GPIO_Mode_FR4 = 0x14,  /*!< 0x14: PxFR4        */
  GPIO_Mode_FR5 = 0x18,  /*!< 0x18: PxFR5        */
  GPIO_Mode_FR6 = 0x1C,  /*!< 0x1C: PxFR6        */
  GPIO_Mode_FR7 = 0x20,  /*!< 0x1C: PxFR7        */
  GPIO_Mode_OD  = 0x28,  /*!< 0x28: PxOD         */
  GPIO_Mode_PUP = 0x2C,  /*!< 0x2C: PxPUP        */
  GPIO_Mode_PDN = 0x30,  /*!< 0x30: PxPDN        */
  GPIO_Mode_IE  = 0x38   /*!< 0x38: PxIE         */
}gpio_mode_t;
 
/** 
 *  @enum gpio_fr_t
 *  @brief    Port Function Number Enumerated Type Definition.
 */
typedef enum
{
  GPIO_FR_1 = 1,   /*!< 1: PxFR1       */
  GPIO_FR_2,       /*!< 2: PxFR2       */
  GPIO_FR_3,       /*!< 3: PxFR3       */
  GPIO_FR_4,       /*!< 4: PxFR4       */
  GPIO_FR_5,       /*!< 5: PxFR5       */
  GPIO_FR_6,       /*!< 6: PxFR6       */
  GPIO_FR_7,       /*!< 7: PxFR7       */
  GPIO_FR_NA,      /*!< 8: N/A         */
  GPIO_FR_Max,      /*!< Max Number     */
}gpio_fr_t;
/** 
 *  @enum gpio_pa0_func_t
 *  @brief    PortA0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PA0_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PA0_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PA0_INT07      = 0x0, /*!< 0: UT0TXDB        */
  GPIO_PA0_UT1TXDA    = 0x1, /*!< 1: UT1TXDA        */
  GPIO_PA0_UT1RXD     = 0x2, /*!< 2: UT1RXD         */
  GPIO_PA0_TSPI1TXD   = 0x3, /*!< 3: TSPI1TXD       */
  GPIO_PA0_T32A01INB0 = 0x4, /*!< 4: T32A01INB0     */
  GPIO_PA0_T32A01OUTB = 0x5, /*!< 5: T32A01OUTB     */
}gpio_pa0_func_t;
 
/** 
 *  @enum gpio_pa1_func_t
 *  @brief    PortA1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PA1_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PA1_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PA1_INT09      = 0x0, /*!< 0: INT09          */
  GPIO_PA1_UT1RXD     = 0x1, /*!< 1: UT1RXD         */
  GPIO_PA1_UT1TXDA    = 0x2, /*!< 2: UT1TXDA        */
  GPIO_PA1_TSPI1RXD   = 0x3, /*!< 3: TSPI1RXD       */
  GPIO_PA1_T32A01INA0 = 0x4, /*!< 4: T32A01INA0     */
  GPIO_PA1_T32A01INC0 = 0x5, /*!< 5: T32A01INC0     */
}gpio_pa1_func_t;
 
/** 
 *  @enum gpio_pa2_func_t
 *  @brief    PortA2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PA2_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PA2_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PA2_TSPI1CLK   = 0x3, /*!< 3: TSPI1CLK       */
  GPIO_PA2_T32A01INA1 = 0x4, /*!< 4: T32A01INA1     */
  GPIO_PA2_T32A01INC1 = 0x5, /*!< 5: T32A01INC1     */
  GPIO_PA2_T32A01OUTA = 0x6, /*!< 6: T32A01OUTA     */
  GPIO_PA2_T32A01OUTC = 0x7, /*!< 7: T32A01OUTC     */
}gpio_pa2_func_t;
 

/** 
 *  @enum gpio_pb0_func_t
 *  @brief    PortB0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PB0_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PB0_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PB0_I2C0SDA    = 0x3, /*!< 3: I2C0SDA        */
  GPIO_PB0_T32A05OUTA = 0x4, /*!< 4: T32A05OUTA     */
  GPIO_PB0_T32A05OUTC = 0x5, /*!< 5: T32A05OUTC     */
  GPIO_PB0_PMD0DBG    = 0x7, /*!< 7: PMD0DBG        */
}gpio_pb0_func_t;
 
/** 
 *  @enum gpio_pb1_func_t
 *  @brief    PortB1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PB1_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PB1_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PB1_I2CSCL     = 0x3, /*!< 3: I2CSCL         */
  GPIO_PB1_T32A05INA0 = 0x4, /*!< 4: T32A05INA0     */
  GPIO_PB1_T32A05INC0 = 0x5, /*!< 5: T32A05INC0     */
  GPIO_PB1_TRGIN1     = 0x6, /*!< 6: TRGIN1         */
  GPIO_PB1_PMD1DBG    = 0x7, /*!< 7: PMD1DBG        */
}gpio_pb1_func_t;
 
/** 
 *  @enum gpio_pc0_func_t
 *  @brief    PortC0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PC0_INPUT      = 0x0, /*!< 0: Input Port   */
  GPIO_PC0_OUTPUT     = 0x0, /*!< 0: Output Port  */
  GPIO_PC0_INT08      = 0x0, /*!< 0: INT08        */
  GPIO_PC0_UT3TXDA    = 0x1, /*!< 1: UT3TXDA      */
  GPIO_PC0_UT3RXD     = 0x2, /*!< 2: UT3RXD       */
  GPIO_PC0_TSPI3TXD   = 0x3, /*!< 3: TSPI3TXD     */
  GPIO_PC0_T32A03OUTA = 0x4, /*!< 4: T32A03OUTA   */
  GPIO_PC0_T32A03OUTC = 0x5, /*!< 5: T32A2OUTC    */
}gpio_pc0_func_t;
 
/** 
 *  @enum gpio_pc1_func_t
 *  @brief    PortC1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PC1_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PC1_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PC1_INT10      = 0x0, /*!< 0: INT10          */
  GPIO_PC1_UT3RXD     = 0x1, /*!< 1: UT3RXD         */
  GPIO_PC1_UT3TXDA    = 0x2, /*!< 2: UT3TXDA        */
  GPIO_PC1_TSPI3RXD   = 0x3, /*!< 3: TSPI3RXD       */
  GPIO_PC1_T32A3INA0  = 0x4, /*!< 4: T32A3INA0      */
  GPIO_PC1_T32A3INC0  = 0x5, /*!< 5: T32A2INC0      */
}gpio_pc1_func_t;
 
/** 
 *  @enum gpio_pc2_func_t
 *  @brief    PortC2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PC2_INPUT     = 0x0, /*!< 0: Input Port     */
  GPIO_PC2_OUTPUT    = 0x0, /*!< 0: Output Port    */
  GPIO_PC2_TSPI3CLK  = 0x3, /*!< 3: TSPI3CLK       */
  GPIO_PC2_T32A3INA1 = 0x4, /*!< 4: T32A3INA1      */
  GPIO_PC2_T32A3INC1 = 0x5, /*!< 5: T32A3INC1      */
  GPIO_PC2_RTCOUT    = 0x6, /*!< 6: RTCOUT         */
}gpio_pc2_func_t;
 
/** 
 *  @enum gpio_pd0_func_t
 *  @brief    PortD0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PD0_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PD0_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PD0_AINA00 = 0x0, /*!< 0: AINA00         */
}gpio_pd0_func_t;
 
/** 
 *  @enum gpio_pd1_func_t
 *  @brief    PortD1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PD1_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PD1_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PD1_AINA01 = 0x0, /*!< 0: AINA01         */
  GPIO_PD1_AINA02 = 0x0, /*!< 0: AINA02         */
}gpio_pd1_func_t;
 
/** 
 *  @enum gpio_pd2_func_t
 *  @brief    PortD2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PD2_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PD2_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PD2_AINA03 = 0x0, /*!< 0: AINA03         */
}gpio_pd2_func_t;
 
/** 
 *  @enum gpio_pd3_func_t
 *  @brief    PortD3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PD3_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PD3_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PD3_AINA04 = 0x0, /*!< 0: AINA04         */
  GPIO_PD3_AINA05 = 0x0, /*!< 0: AINA05         */
}gpio_pd3_func_t;

/** 
 *  @enum gpio_pd4_func_t
 *  @brief    PortD4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PD4_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PD4_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PD4_AINA06 = 0x0, /*!< 0: AINA06         */
}gpio_pd4_func_t;

/** 
 *  @enum gpio_pd5_func_t
 *  @brief    PortD5 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PD5_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PD5_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PD5_AINA07 = 0x0, /*!< 0: AINA07         */
  GPIO_PD5_AINA08 = 0x0, /*!< 0: AINA08         */
}gpio_pd5_func_t;

/** 
 *  @enum gpio_pd6_func_t
 *  @brief    PortD6 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PD6_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PD6_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PD6_AINA09 = 0x0, /*!< 0: AINA09         */
}gpio_pd6_func_t;


/** 
 *  @enum gpio_pe0_func_t
 *  @brief    PortE0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE0_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PE0_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PE0_AINA10 = 0x0, /*!< 0: AINA10         */
}gpio_pe0_func_t;
 
/** 
 *  @enum gpio_pe1_func_t
 *  @brief    PortE1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE1_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PE1_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PE1_AINA11 = 0x0, /*!< 0: AINA11         */
}gpio_pe1_func_t;
 
/** 
 *  @enum gpio_pe2_func_t
 *  @brief    PortE2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE2_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PE2_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PE2_AINA12 = 0x0, /*!< 0: AINA12         */
}gpio_pe2_func_t;
 
 /** 
 *  @enum gpio_pe3_func_t
 *  @brief    PortE3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE3_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PE3_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PE3_AINA13 = 0x0, /*!< 0: AINA13         */
}gpio_pe3_func_t;

 /** 
 *  @enum gpio_pe4_func_t
 *  @brief    PortE4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE4_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PE4_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PE4_AINA14 = 0x0, /*!< 0: AINA14         */
}gpio_pe4_func_t;
 
/** 
 *  @enum gpio_pe5_func_t
 *  @brief    PortE5 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE5_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PE5_OUTPUT = 0x0, /*!< 0: Output Port    */
  GPIO_PE5_AINA15 = 0x0, /*!< 0: AINA15         */
  GPIO_PE5_VREFH  = 0x0, /*!< 0: VREFH          */
}gpio_pe5_func_t;

/** 
 *  @enum gpio_pf0_func_t
 *  @brief    PortF0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PF0_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PF0_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PF0_T32A04OUTA = 0x4, /*!< 4: T32A04OUTA     */
  GPIO_PF0_T32A04OUTC = 0x5, /*!< 5: T32A04OUTC     */
  GPIO_PF0_TRGIN0     = 0x6, /*!< 6: TRGIN0         */
  GPIO_PF0_EMG1_N     = 0x7, /*!< 7: EMG1_N         */
}gpio_pf0_func_t;
 
/** 
 *  @enum gpio_pf1_func_t
 *  @brief    PortF1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PF1_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PF1_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PF1_T32A04INA0 = 0x4, /*!< 4: T32A04INA0     */
  GPIO_PF1_T32A04INC0 = 0x5, /*!< 5: T32A04INC0     */
}gpio_pf1_func_t;

/** 
 *  @enum gpio_tpf2_func_t
 *  @brief    PortTPF2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_TPF2_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_TPF2_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_TPF2_T32A04INA1 = 0x4, /*!< 4: T32A04INA1     */
  GPIO_TPF2_T32A04INC1 = 0x5, /*!< 5: T32A04INC1     */
  GPIO_TPF2_TRGIN2     = 0x6, /*!< 6: TRGIN2         */
}gpio_tpf2_func_t;

/** 
 *  @enum gpio_pg0_func_t
 *  @brief    PortG0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PG0_INPUT     = 0x0, /*!< 0: Input Port     */
  GPIO_PG0_OUTPUT    = 0x0, /*!< 0: Output Port    */
  GPIO_PG0_INT04     = 0x0, /*!< 0: INT04          */
  GPIO_PG0_UT2TXDA   = 0x1, /*!< 1: UT2TXDA        */
  GPIO_PG0_TSPI2TXD  = 0x2, /*!< 2: TSPI2TXD       */
  GPIO_PG0_T32A2OUTA = 0x3, /*!< 3: T32A2OUTA      */
  GPIO_PG0_T32A2OUTC = 0x4, /*!< 4: T32A2OUTC      */
  GPIO_PG0_ENC0A     = 0x5, /*!< 5: ENC0A          */
  GPIO_PG0_UO1       = 0x6, /*!< 6: UO1            */
  GPIO_PG0_PMG0DBG   = 0x7, /*!< 7: PMG0DBG        */
}gpio_pg0_func_t;

/** 
 *  @enum gpio_pg1_func_t
 *  @brief    PortG1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PG1_INPUT     = 0x0, /*!< 0: Input Port     */
  GPIO_PG1_OUTUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PG1_INT05     = 0x0, /*!< 0: INT05          */
  GPIO_PG1_UT2RXD    = 0x1, /*!< 1: UT2RXD         */
  GPIO_PG1_TSPI2RXD  = 0x2, /*!< 2: TSPI2RXD       */
  GPIO_PG1_T32A2INA0 = 0x3, /*!< 3: T32A2INA0      */
  GPIO_PG1_T32A2INC0 = 0x4, /*!< 4: T32A2INC0      */
  GPIO_PG1_ENC0B     = 0x5, /*!< 5: ENC0B          */
  GPIO_PG1_VO1       = 0x6, /*!< 6: VO1            */
  GPIO_PG1_PMD1DBG   = 0x7, /*!< 7: PMD1DBG        */
}gpio_pg1_func_t;

/** 
 *  @enum gpio_pg2_func_t
 *  @brief    PortG2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PG2_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PG2_OUTUT      = 0x0, /*!< 0: Output Port    */
  GPIO_PG2_TSPI2CLK   = 0x2, /*!< 2: TSPI2CLK       */
  GPIO_PG2_T32A02INA0 = 0x3, /*!< 3: T32A02INA0     */
  GPIO_PG2_T32A02INC0 = 0x4, /*!< 4: T32A02INC0     */
  GPIO_PG2_ENC0Z      = 0x5, /*!< 5: ENC0Z          */
  GPIO_PG2_WO1        = 0x6, /*!< 6: WO1            */
}gpio_pg2_func_t;

/** 
 *  @enum gpio_pg3_func_t
 *  @brief    PortG3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PG3_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PG3_OUTUT      = 0x0, /*!< 0: Output Port    */
  GPIO_PG3_XO1        = 0x6, /*!< 6: XO1            */
  GPIO_PG3_COMOUT0    = 0x7, /*!< 7: COMOUT0        */
}gpio_pg3_func_t;

/** 
 *  @enum gpio_pg4_func_t
 *  @brief    PortG4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PG4_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PG4_OUTUT      = 0x0, /*!< 0: Output Port    */
  GPIO_PG4_YO1        = 0x6, /*!< 6: YO1            */
  GPIO_PG4_COMOUT1    = 0x7, /*!< 7: COMOUT1        */
}gpio_pg4_func_t;

/** 
 *  @enum gpio_pg5_func_t
 *  @brief    PortG5 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PG5_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PG5_OUTUT      = 0x0, /*!< 0: Output Port    */
  GPIO_PG5_ZO1        = 0x6, /*!< 6: ZO1            */
  GPIO_PG5_COMOUT2    = 0x7, /*!< 7: COMOUT2        */
}gpio_pg5_func_t;

/** 
 *  @enum gpio_ph0_func_t
 *  @brief    PortH0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PH0_INPUT     = 0x0, /*!< 0: Input Port     */
  GPIO_PH0_X1        = 0x0, /*!< 0: X1             */
}gpio_ph0_func_t;
 
/** 
 *  @enum gpio_ph1_func_t
 *  @brief    PortH1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PH1_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PH1_X2     = 0x0, /*!< 0: X2             */
}gpio_ph1_func_t;
 
/** 
 *  @enum gpio_ph2_func_t
 *  @brief    PortH2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PH2_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PH2_XT1    = 0x0, /*!< 0: XT1            */
}gpio_ph2_func_t;
 
 /** 
 *  @enum gpio_ph3_func_t
 *  @brief    PortH3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PH3_INPUT  = 0x0, /*!< 0: Input Port     */
  GPIO_PH3_XT2    = 0x0, /*!< 0: XT2            */
}gpio_ph3_func_t;

/** 
 *  @enum gpio_pj0_func_t
 *  @brief    PortJ0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PJ0_INPUT    = 0x0, /*!< 0: Input Port     */
  GPIO_PJ0_OUTPUT   = 0x0, /*!< 0: Output Port    */
  GPIO_PJ0_UO0      = 0x5, /*!< 5: UO0            */
  GPIO_PJ0_SCOUT    = 0x6, /*!< 6: SCOUT          */
  GPIO_PJ0_PMD0DBG  = 0x7, /*!< 7: PMD0DBG        */
}gpio_pj0_func_t;
 
/** 
 *  @enum gpio_pj1_func_t
 *  @brief    PortJ1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PJ1_INPUT     = 0x0, /*!< 0: Input Port     */
  GPIO_PJ1_OUTPUT    = 0x0, /*!< 0: Output Port    */
  GPIO_PJ1_XO0       = 0x5, /*!< 5: XO0            */
  GPIO_PJ1_PMD1DBG   = 0x7, /*!< 7: PMD1DBG        */
}gpio_pj1_func_t;
 
/** 
 *  @enum gpio_pj2_func_t
 *  @brief    PortJ2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PJ2_INPUT     = 0x0, /*!< 0: Input Port     */
  GPIO_PJ2_OUTPUT    = 0x0, /*!< 0: Output Port    */
  GPIO_PJ2_VO0       = 0x5, /*!< 5: VO0         */
}gpio_pj2_func_t;
 
 /** 
 *  @enum gpio_pj3_func_t
 *  @brief    PortJ3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PJ3_INPUT       = 0x0, /*!< 0: Input Port     */
  GPIO_PJ3_OUTPUT      = 0x0, /*!< 0: Output Port    */
  GPIO_PJ3_YO0         = 0x5, /*!< 5: YO0             */
}gpio_pj3_func_t;

 /** 
 *  @enum gpio_pj4_func_t
 *  @brief    PortJ4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PJ4_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PJ4_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PJ4_WO0        = 0x5, /*!< 5: WO0             */
}gpio_pj4_func_t;
 
/** 
 *  @enum gpio_pj5_func_t
 *  @brief    PortJ5 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PJ5_INPUT     = 0x0, /*!< 0: Input Port     */
  GPIO_PJ5_OUTPUT    = 0x0, /*!< 0: Output Port    */
  GPIO_PJ5_ZO0       = 0x5, /*!< 5: ZO0            */
}gpio_pj5_func_t;

/** 
 *  @enum gpio_pj6_func_t
 *  @brief    PortJ6 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PJ6_INPUT     = 0x0, /*!< 0: Input Port     */
  GPIO_PJ6_OUTPUT    = 0x0, /*!< 0: Output Port    */
  GPIO_PJ6_EMG0_N    = 0x5, /*!< 5: MG0_N          */
}gpio_pj6_func_t;

/** 
 *  @enum gpio_pj7_func_t
 *  @brief    PortJ7 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PJ7_INPUT     = 0x0, /*!< 0: Input Port     */
  GPIO_PJ7_OUTPUT    = 0x0, /*!< 0: Output Port    */
  GPIO_PJ7_OVV0_N    = 0x5, /*!< 5: OVV0_N         */
}gpio_pj7_func_t;
/** 
 *  @enum gpio_pk0_func_t
 *  @brief    PortK0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PK0_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PK0_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PK0_INT00      = 0x0, /*!< 0: INT00          */
  GPIO_PK0_UT0RXD     = 0x1, /*!< 1: UT0RXD         */
  GPIO_PK0_IT0TXDA    = 0x2, /*!< 2: IT0TXDA        */
  GPIO_PK0_T32A00OUTA = 0x4, /*!< 4: T32A00OUTA     */
  GPIO_PK0_T32A00OUTC = 0x5, /*!< 5: T32A00OUTC     */
  GPIO_PK0_TD1        = 0x7, /*!< 7: TD1            */
}gpio_pk0_func_t;
 
/** 
 *  @enum gpio_pk1_func_t
 *  @brief    PortK1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PK1_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PK1_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PK1_INT01      = 0x0, /*!< 0: INT01          */
  GPIO_PK1_UT0TXDA    = 0x1, /*!< 1: UT0TXDA        */
  GPIO_PK1_UT0RXD     = 0x2, /*!< 2: UT0RXD         */
  GPIO_PK1_T32A00INA0 = 0x4, /*!< 5: T32A00INA0     */
  GPIO_PK1_T32A00INC0 = 0x5, /*!< 5: T32A00INC0     */
  GPIO_PK1_TDO_SWV    = 0x7, /*!< 7: TDO_SWV        */
}gpio_pk1_func_t;
 
/** 
 *  @enum gpio_pk2_func_t
 *  @brief    PortK2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PK2_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PK2_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PK2_INT02      = 0x0, /*!< 0: INT02          */
  GPIO_PK2_UT0RXD     = 0x1, /*!< 1: UT0RXD         */
  GPIO_PK2_UT0TXDA    = 0x2, /*!< 2: UT0TXDA        */
  GPIO_PK2_TSPI0RXD   = 0x3, /*!< 3: TSPI0RXD       */
  GPIO_PK2_TMS_SWDIO  = 0x7, /*!< 4: TMS_SWDIO      */
}gpio_pk2_func_t;
 
 /** 
 *  @enum gpio_pk3_func_t
 *  @brief    PortK3 Function Enumerated Type Definition.
 */
typedef enum
{ 
  GPIO_PK3_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PK3_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PK3_INT03      = 0x0, /*!< 0: INT03          */
  GPIO_PK3_UT0TXDA    = 0x1, /*!< 1: UT0TXDA        */
  GPIO_PK3_UT0RXD     = 0x2, /*!< 2: UT0RXD         */
  GPIO_PK3_TSPI0TXD   = 0x3, /*!< 3: TSPI0TXD       */
  GPIO_PK3_TCK_SWCLK  = 0x7, /*!< 7: TCK_SWCLK      */
}gpio_pk3_func_t;

 /** 
 *  @enum gpio_pk4_func_t
 *  @brief    PortK4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PK4_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PK4_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PK4_TSPI0CLK   = 0x3, /*!< 3: TSPI0CLK       */
  GPIO_PK4_NBDSYNC    = 0x6, /*!< 6: NBDSYNC        */
  GPIO_PK4_TRST_N     = 0x7, /*!< 7: TRST_N         */
}gpio_pk4_func_t;

/** 
 *  @enum gpio_pl0_func_t
 *  @brief    PortL0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL0_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PL0_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PL0_NBDDATA0   = 0x6, /*!< 6: NBDDATA0       */
  GPIO_PL0_TRACEDATA0 = 0x7, /*!< 7: TRACEDATA0     */
}gpio_pl0_func_t;
 
/** 
 *  @enum gpio_pl1_func_t
 *  @brief    PortL1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL1_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PL1_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PL1_NBDDATA1   = 0x6, /*!< 6: NBDDATA1       */
  GPIO_PL1_TRACEDATA1 = 0x7, /*!< 7: TRACEDATA1     */
}gpio_pl1_func_t;
 
/** 
 *  @enum gpio_pl2_func_t
 *  @brief    PortL2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL2_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PL2_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PL2_NBDDATA2   = 0x6, /*!< 6: NBDDATA2       */
  GPIO_PL2_TRACEDATA2 = 0x7, /*!< 7: TRACEDATA2       */
}gpio_pl2_func_t;
 
 /** 
 *  @enum gpio_pl3_func_t
 *  @brief    PortL3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL3_INPUT      = 0x0, /*!< 0: Input Port     */
  GPIO_PL3_OUTPUT     = 0x0, /*!< 0: Output Port    */
  GPIO_PL3_NBDDATA3   = 0x6, /*!< 6: NBDDATA3          */
  GPIO_PL3_TRACEDATA3 = 0x7, /*!< 7: TRACEDATA3      */
}gpio_pl3_func_t;

 /** 
 *  @enum gpio_pl4_func_t
 *  @brief    PortL4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL4_INPUT    = 0x0, /*!< 0: Input Port     */
  GPIO_PL4_OUTPUT   = 0x0, /*!< 0: Output Port    */
  GPIO_PL4_NBDCLK   = 0x6, /*!< 6: NBDCLK          */
  GPIO_PL4_TRACECLK = 0x7, /*!< 7: TRACECLK          */
}gpio_pl4_func_t;
 
/**
 *  @} 
 */ /* End of group GPIO_Exported_Typedef */
 
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup GPIO_Exported_Typedef GPIO Exported Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  GPIO handle structure definenition.
*/
/*----------------------------------*/
#if defined(TMPM4K4A)
typedef struct gpio_pa_handle
{  
    TSB_PA_TypeDef       *p_pa_instance;     /*!< Registers base address.           */
    TSB_PB_TypeDef       *p_pb_instance;     /*!< Registers base address.           */
    TSB_PC_TypeDef       *p_pc_instance;     /*!< Registers base address.           */
    TSB_PD_TypeDef       *p_pd_instance;     /*!< Registers base address.           */
    TSB_PE_TypeDef       *p_pe_instance;     /*!< Registers base address.           */
    TSB_PF_TypeDef       *p_pf_instance;     /*!< Registers base address.           */
    TSB_PG_TypeDef       *p_pg_instance;     /*!< Registers base address.           */
    TSB_PH_TypeDef       *p_ph_instance;     /*!< Registers base address.           */
    TSB_PJ_TypeDef       *p_pj_instance;     /*!< Registers base address.           */
    TSB_PK_TypeDef       *p_pk_instance;     /*!< Registers base address.           */
    TSB_PL_TypeDef       *p_pl_instance;     /*!< Registers base address.           */
} gpio_t;
#endif  /* TMPM4K4A */
#if defined(TMPM4K2A)
typedef struct gpio_pa_handle
{  
    TSB_PA_TypeDef       *p_pa_instance;     /*!< Registers base address.           */
    TSB_PB_TypeDef       *p_pb_instance;     /*!< Registers base address.           */
    TSB_PC_TypeDef       *p_pc_instance;     /*!< Registers base address.           */
    TSB_PD_TypeDef       *p_pd_instance;     /*!< Registers base address.           */
    TSB_PE_TypeDef       *p_pe_instance;     /*!< Registers base address.           */
    TSB_PF_TypeDef       *p_pf_instance;     /*!< Registers base address.           */
    TSB_PG_TypeDef       *p_pg_instance;     /*!< Registers base address.           */
    TSB_PH_TypeDef       *p_ph_instance;     /*!< Registers base address.           */
    TSB_PJ_TypeDef       *p_pj_instance;     /*!< Registers base address.           */
    TSB_PK_TypeDef       *p_pk_instance;     /*!< Registers base address.           */
} gpio_t;
#endif  /* TMPM4K2A */
#if defined(TMPM4K1A)
typedef struct gpio_pa_handle
{  
    TSB_PA_TypeDef       *p_pa_instance;     /*!< Registers base address.           */
    TSB_PB_TypeDef       *p_pb_instance;     /*!< Registers base address.           */
    TSB_PC_TypeDef       *p_pc_instance;     /*!< Registers base address.           */
    TSB_PD_TypeDef       *p_pd_instance;     /*!< Registers base address.           */
    TSB_PE_TypeDef       *p_pe_instance;     /*!< Registers base address.           */
    TSB_PF_TypeDef       *p_pf_instance;     /*!< Registers base address.           */
    TSB_PG_TypeDef       *p_pg_instance;     /*!< Registers base address.           */
    TSB_PH_TypeDef       *p_ph_instance;     /*!< Registers base address.           */
    TSB_PJ_TypeDef       *p_pj_instance;     /*!< Registers base address.           */
    TSB_PK_TypeDef       *p_pk_instance;     /*!< Registers base address.           */
} gpio_t;
#endif  /* TMPM4K1A */
#if defined(TMPM4K0A)
typedef struct gpio_pa_handle
{  
    TSB_PD_TypeDef       *p_pd_instance;     /*!< Registers base address.           */
    TSB_PE_TypeDef       *p_pe_instance;     /*!< Registers base address.           */
    TSB_PG_TypeDef       *p_pg_instance;     /*!< Registers base address.           */
    TSB_PH_TypeDef       *p_ph_instance;     /*!< Registers base address.           */
    TSB_PJ_TypeDef       *p_pj_instance;     /*!< Registers base address.           */
    TSB_PK_TypeDef       *p_pk_instance;     /*!< Registers base address.           */
} gpio_t;
#endif  /* TMPM4K0A */
/**
 *  @}
 */ /* End of group GPIO_Exported_Typedef */
 
/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup GPIO_Exported_functions GPIO Exported Functions
 *  @{
 */
TXZ_Result gpio_init(gpio_t *p_obj, uint32_t group);
TXZ_Result gpio_deinit(gpio_t *p_obj, uint32_t group);
TXZ_Result gpio_func(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t func, uint32_t inout);
TXZ_Result gpio_write_mode(gpio_t *p_obj, uint32_t group, uint32_t mode, uint32_t val);
TXZ_Result gpio_read_mode(gpio_t *p_obj, uint32_t group, uint32_t mode, uint32_t *val);
TXZ_Result gpio_write_bit(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t mode, uint32_t val);
TXZ_Result gpio_read_bit(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t mode, gpio_pinstate_t *pinstate);
/**
 *  @}
 */ /* End of group GPIO_Exported_functions */
 
/**
 *  @}
 */ /* End of group GPIO */
 
/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __GPIO_H */
