/**
 *******************************************************************************
 * @file    txz_adc_ex.c
 * @brief   This file provides API functions for ADC driver.
 * @brief   Extension Class.
 * @version V1.0.0.0
 * $Date:: 2017-10-20 19:58:50 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_adc_include.h"
#include "txz_adc_ex.h"

#if defined(__ADC_EX_H)
/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup ADC
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_define ADC Private Define
 *  @{
 */


/** 
 *  @}
 */ /* End of group ADC_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_define ADC Private Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ADC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_define ADC Private Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ADC_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_typedef ADC Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ADC_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_fuctions ADC Private Fuctions
 *  @{
 */

#ifdef DEBUG
    __STATIC_INLINE int32_t check_param_interrupt(uint32_t param);
    __STATIC_INLINE int32_t check_param_type(uint32_t param);
#endif

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Check the Interrupt's parameter.
  * @param  param    :Interrupt's parameter
  * @retval ADC_PARAM_OK :Valid
  * @retval ADC_PARAM_NG :Invalid
  * @note   Macro definition is @ref ADC_IntEnable"ADC_INT_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interrupt(uint32_t param)
{
    int32_t result = ADC_PARAM_NG;

    switch (param) 
    {
    case ADC_INT_DISABLE:
    case ADC_INT_ENABLE:
        result = ADC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the Conversion Type's parameter.
  * @param  param    :Conversion Type's parameter
  * @retval ADC_PARAM_OK :Valid
  * @retval ADC_PARAM_NG :Invalid
  * @note   Macro definition is @ref ADC_Conversion"ADC_CONVERSION_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_type(uint32_t param)
{
    int32_t result = ADC_PARAM_NG;

    switch (param) 
    {
    case ADC_CONVERSION_DISABLE:
    case ADC_CONVERSION_CNT:
    case ADC_CONVERSION_SGL:
    case ADC_CONVERSION_TRG:
        result = ADC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}
#endif

/**
 *  @}
 */ /* End of group ADC_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the ADC object.
  * @param     p_obj       :ADC object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  * @attention After initialization, 3us of stabilization time is needed.
  */
/*--------------------------------------------------*/
TXZ_Result adc_ex_init(adc_ex_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    /* Check the parameter.           */
    /* Non. */
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Construct Superclass        */
    /*------------------------------*/
    if (adc_init(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*------------------------------*/
    /*  Construct Subclass          */
    /*------------------------------*/
    /* now, no processing */

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Release the ADC object.
  * @param     p_obj       :ADC object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_ex_deinit(adc_ex_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Destruct Subclass           */
    /*------------------------------*/
    /* now, no processing */
    /*------------------------------*/
    /*  Destruct Superclass         */
    /*------------------------------*/
    if (adc_deinit(&p_obj->super) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     ADC Channel Setting.
  * @param     p_obj       :ADC object.
  * @param     ch          :Channel. Range is (value < ADC_NUM_MAX).
  * @param     p_setting   :Channel Setting Source Address.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @pre       Conversion has stoped.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_ex_channel_setting(adc_ex_t *p_obj, uint32_t ch, adc_ex_channel_setting_t *p_setting)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->super.p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_setting));
    /* Check the parameter.           */
    assert_param(check_param_channel(ch, ADC_NUM_MAX));
    assert_param(check_param_interrupt(p_setting->interrupt));
    assert_param(check_param_type(p_setting->type));
    assert_param(check_param_ain(p_setting->ain, ADC_AIN_RANGE_MIN, ADC_AIN_RANGE_MAX));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Channel Class Construct     */
    /*------------------------------*/
    {
         adc_ch_t *p_ch = &p_obj->super.info.ch[ch];

         p_ch->p_tset   = (__IO uint32_t *)(&p_obj->super.p_instance->TSET0 + ch);
         p_ch->p_reg    = (__I uint32_t *)(&p_obj->super.p_instance->REG0 + ch);
         p_ch->init.interrupt   = p_setting->interrupt;
         p_ch->init.type        = p_setting->type;
         p_ch->init.ain         = p_setting->ain;
         result = adc_ch_init(p_ch);
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     ADC Channel Clear.
  * @param     p_obj       :ADC object.
  * @param     ch          :Channel. Range is (value < ADC_NUM_MAX).
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @pre       Conversion has stoped.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_ex_channel_clear(adc_ex_t *p_obj, uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    /* Check the parameter.           */
    /* Non */
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Use Superclass              */
    /*------------------------------*/
    result = adc_channel_clear(&p_obj->super, ch);

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Get AD value.
  * @param  p_obj       :ADC object.
  * @param  ch          :Channel. Range is (value < ADC_NUM_MAX).
  * @param  p_value     :AD value. Destination address.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result adc_ex_channel_get_value(adc_ex_t *p_obj, uint32_t ch, uint32_t *p_value)
{
    TXZ_Result result = TXZ_ERROR;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->super.p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_value));
    /* Check the parameter.           */
    assert_param(check_param_channel(ch, ADC_NUM_MAX));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Get Value                   */
    /*------------------------------*/
    {
         adc_ch_t *p_ch = &p_obj->super.info.ch[ch];

         result = adc_ch_get_value(p_ch, p_value);
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Start Conversion.
  * @param  p_obj       :ADC object.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  * @pre    Conversion has stoped.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_ex_start(adc_ex_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->super.p_instance));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Enable Conversion           */
    /*------------------------------*/
    {
        uint32_t i;
        adc_ch_t *p_ch;
        uint32_t cr0 = (ADxCR0_ADEN_ENABLE | ADxCR0_CNT_DISABLE);
        uint32_t cr1 = (ADxCR1_CNTDMEN_DISABLE | ADxCR1_SGLDMEN_DISABLE | ADxCR1_TRGDMEN_DISABLE | ADxCR1_TRGEN_DISABLE);

        for (i=0; i<ADC_NUM_MAX; i++)
        {
            p_ch = &p_obj->super.info.ch[i];
            switch (p_ch->init.type)
            {
            case ADC_CONVERSION_CNT:
                cr0 |= ADxCR0_CNT_ENABLE;
                break;
            case ADC_CONVERSION_SGL:
                cr0 |= ADxCR0_SGL_ENABLE;
                break;
            case ADC_CONVERSION_TRG:
                cr1 |= ADxCR1_TRGEN_ENABLE;
                break;
            default:
                /* no processing */
                break;
            }
        }
        /*--- ADxCR1  ---*/
        p_obj->super.p_instance->CR1 = cr1;
        /*--- ADxCR0  ---*/
        p_obj->super.p_instance->CR0 = cr0;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Stop Conversion.
  * @param  p_obj       :ADC object.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_ex_stop(adc_ex_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Use Superclass              */
    /*------------------------------*/
    result = adc_stop(&p_obj->super);

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Get Conversion Status.
  * @details  Status bits.
  * | Bit   | Bit Symbol | Function                                                           |
  * | :---  | :---       | :---                                                               |
  * | 31-8  | -          | -                                                                  |
  * | 7     | ADBF       | AD Running Flag. Use @ref ADC_Status.                              |
  * | 6-4   | -          | -                                                                  |
  * | 3     | CNTF       | Continuity Conversion Running Flag. Use @ref ADC_ContinuityStatus. |
  * | 2     | SNGF       | Single Conversion Running Flag. Use @ref ADC_SingleStatus.         |
  * | 1     | TRGF       | Trigger Conversion Running Flag. Use @ref ADC_TriggerStatus.       |
  * | 0     | -          | -                                                                  |
  * 
  * @param  p_obj       :ADC object.
  * @param  p_status    :Conversion Status. Destination address.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result adc_ex_get_status(adc_ex_t *p_obj, uint32_t *p_status)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->super.p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_status));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Read Register               */
    /*------------------------------*/
    *p_status = p_obj->super.p_instance->ST;

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  IRQ Handler for single conversion done.
  * @param  p_obj  :ADC object.
  * @retval -
  * @note   Call by Single Conversion Done IRQ Handler.
 */
/*--------------------------------------------------*/
void adc_ex_single_irq_handler(adc_ex_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Call Handler                */
    /*------------------------------*/
    if (p_obj->handler.single != ADC_NULL)
    {
        p_obj->handler.single(p_obj->super.init.id, TXZ_SUCCESS);
    }
}

/*--------------------------------------------------*/
/**
  * @brief  IRQ Handler for continuity conversion done.
  * @param  p_obj  :ADC object.
  * @retval -
  * @note   Call by Continuity Conversion Done IRQ Handler.
 */
/*--------------------------------------------------*/
void adc_ex_continuity_irq_handler(adc_ex_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Call Handler                */
    /*------------------------------*/
    if (p_obj->handler.continuity != ADC_NULL)
    {
        p_obj->handler.continuity(p_obj->super.init.id, TXZ_SUCCESS);
    }
}

/*--------------------------------------------------*/
/**
  * @brief  IRQ Handler for trigger conversion done.
  * @param  p_obj  :ADC object.
  * @retval -
  * @note   Call by Trigger Conversion Done IRQ Handler.
 */
/*--------------------------------------------------*/
void adc_ex_trigger_irq_handler(adc_ex_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef DEBUG
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef DEBUG */
    /*------------------------------*/
    /*  Call Handler                */
    /*------------------------------*/
    if (p_obj->handler.trigger != ADC_NULL)
    {
        p_obj->handler.trigger(p_obj->super.init.id, TXZ_SUCCESS);
    }
}
/**
 *  @}
 */ /* End of group ADC_Exported_functions */

/**
 *  @} 
 */ /* End of group ADC */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__ADC_EX_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

