/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ADC UART Sample Application.
 * @version V1.0.0.2
 * $Date:: 2018-07-12 11:00:50 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "bsp_timer.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup ADC_UART ADC_UART Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_macro ADC_UART Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group ADC_UART_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_define ADC_UART Private Define
 *  @{
 */

/*----------------------*/
/*  ADC Setting         */
/*----------------------*/
/** 
 *  @defgroup ADC_Setting ADC Setting
 *  @{
 */
#define CFG_ADC_WAIT_TIME             ((uint32_t)1000)    /*!< Conversion Wait Time(tick) */
/** 
 *  @}
 */ /* End of group ADC_Setting */

/*----------------------*/
/*  Output Setting      */
/*----------------------*/
/** 
 *  @defgroup Output_Setting Output Setting
 *  @{
 */
#define CFG_OUTPUT_INTERVAL           ((uint32_t)5000)   /*!< deg.C output period(ms) */
/** 
 *  @}
 */ /* End of group Output_Setting */

/** 
 *  @}
 */ /* End of group ADC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_define ADC_UART Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)     /*!< Null Pointer  */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */

/** 
 *  @}
 */ /* End of group ADC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_define ADC_UART Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,          /*!< Idle.             */
    PROC_START,         /*!< Start.            */
    PROC_STOP,          /*!< Stop.             */
    PROC_INTERVAL_WAIT, /*!< Interval Wait.    */
    PROC_CONVERSION,    /*!< Conversion.       */
    PROC_OUTPUT,        /*!< Output.           */
    PROC_ERROR          /*!< Error.            */
} Processing;

/*----------------------------------*/
/** 
 * @enum   CoversionState
 * @brief  Conversion Status
*/
/*----------------------------------*/
typedef enum {
    CONVERSION_IDLE = 0U,     /*!< Idle                   */
    CONVERSION_RUN,           /*!< Running.               */
    CONVERSION_SUCCESS,       /*!< Success.               */
    CONVERSION_FAILURE        /*!< Failure.               */
} CoversionState;
/** 
 *  @}
 */ /* End of group ADC_UART_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_typedef ADC_UART Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    t32a_t      t32a;              /*!< Driver      :T32A.     */
    adc_t       adc;               /*!< Driver      :ADC.      */
    adc_ex_t   adc_ex;               /*!< Driver      :ADC.      */
    uart_io_t   *p_uart;           /*!< Application :USB UART. */
    timer_t     timer;             /*!< Application :Timer (1ms timer). */
} instance_t;

/*----------------------------------*/
/** 
 * @brief Information.
*/
/*----------------------------------*/
typedef struct
{
   TXZ_FunctionalState update;     /*!< Update flag.       */
   uint32_t            count;      /*!< Update flag.       */
} information_t;
/**
 *  @}
 */ /* End of group ADC_UART_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_variables ADC_UART Private Variables
 *  @{
 */
static instance_t instance;     /*!< Instance.                */
static Processing proc;         /*!< Processing mode.         */
volatile static CoversionState state;  /*!< Conversion Status */
static information_t info;      /*!< Clock information.       */
static uint32_t data[BSP_POTENTIOMETERS_MAX];  /*!< ADC result Store. */

/**
 *  @}
 */ /* End of group ADC_UART_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Private_fuctions ADC_UART Private Fuctions
 *  @{
 */
static void adc_done_handler(uint32_t id, TXZ_Result result);
static void variable_initialize(void);
static TXZ_Result driver_initialize(void);
static TXZ_Result driver_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void timer_interval_handler(uint32_t id);
/*--------------------------------------------------*/
/** 
  * @brief   Conversion Finish handler
  * @param  id    :  user ID
  * @param  result: result
  * @retval -
  * @note   For Continuous Conversion
  */
/*--------------------------------------------------*/
static void adc_done_handler(uint32_t id, TXZ_Result result)
{
    /*----------------------*/
    /* AD Convert Value get */
    /*----------------------*/
    if (result == TXZ_SUCCESS)
    {
        state = CONVERSION_SUCCESS;
    }
    else
    {
        state = CONVERSION_FAILURE;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.adc.p_instance     = MAIN_NULL;
    instance.p_uart             = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    /* Processing mode. */
    proc = PROC_START;
    /* Information. */
    info.update = TXZ_ENABLE;  /* Init Output. */
    info.count  = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver initialize will process within USB UART. */
    /*----------------------*/
    /* ADC                  */
    /*----------------------*/
    {
        adc_ex_t *p_adc_ex = &instance.adc_ex;

        /*--- Construct  ---*/
        /* Register Allocation */
        p_adc_ex->super.p_instance       = TSB_ADA;
        /*--- Initialize ---*/
        p_adc_ex->super.init.id            = (uint32_t)p_adc_ex;
        p_adc_ex->super.init.convert_time  = ADC_CONVERT_TIME_0_50_AVDD_4_5;
        p_adc_ex->super.init.refbsel       = ADC_REFBSEL_AIN;
        p_adc_ex->super.init.rcut          = ADC_RCUT_NORMAL;
        p_adc_ex->super.handler            = MAIN_NULL;
        /* EX Class Initialize */
        p_adc_ex->handler.single     = adc_done_handler;
        p_adc_ex->handler.continuity = MAIN_NULL;
        p_adc_ex->handler.trigger    = MAIN_NULL;
        if (adc_ex_init(p_adc_ex) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* ADC                  */
    /*----------------------*/
    {
        adc_ex_t *p_adc_ex = &instance.adc_ex;

        /*--- Finalize ---*/
        if (adc_ex_deinit(p_adc_ex) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
        /*--- Destruct ---*/
        /* Register Release */
        p_adc_ex->super.p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver finalize has done within USB UART.    */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Potentiometers               */
    /*----------------------*/
    {
        BSPPotentiometers i;
        adc_ex_t *p_adc_ex = &instance.adc_ex;
        adc_ex_channel_setting_t param;
        for (i=BSP_POTENTIOMETERS_0; i<BSP_POTENTIOMETERS_MAX; i++)
        {
            if (i == (uint32_t)(BSP_POTENTIOMETERS_MAX-1))
            {
                param.interrupt = ADC_INT_ENABLE;
            }
            else
            {
                param.interrupt = ADC_INT_DISABLE;
            }
            param.type      = ADC_CONVERSION_SGL;
            param.ain       = bsp_get_potentiometers_adin(i);
            if (adc_ex_channel_setting(p_adc_ex, bsp_get_potentiometers_ch(i), &param) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    BSPPotentiometers i;

    /*----------------------*/
    /* Thermistors          */
    /*----------------------*/
    if (instance.adc_ex.super.p_instance != MAIN_NULL)
    {
        for (i=BSP_POTENTIOMETERS_0; i<BSP_POTENTIOMETERS_MAX; i++)
        {
            if (adc_ex_channel_clear(&instance.adc_ex, bsp_get_potentiometers_ch(i)) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    if (instance.p_uart != MAIN_NULL)
    {
        uart_io_finalize();
    }
    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    if (instance.timer.init.p_t32a != MAIN_NULL)
    {
        timer_finalize(&instance.timer);
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        /* Increment Tick. Tick is used to driver's Timeout. */
        hal_inc_tick();
        /* Interval Time Count. */
        info.count++;
        if (info.count >= CFG_OUTPUT_INTERVAL)
        {
            /*>>> Critical information. */
            info.update = TXZ_ENABLE;
            /*<<< Critical information. */
            info.count  = 0;
        }
    }
}
/**
 *  @}
 */ /* End of group ADC_UART_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_UART_Exported_functions ADC_UART Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                timer_start(&instance.timer);
                /*----------------------*/
                /* Interrupt Enable     */
                /*----------------------*/
                NVIC_EnableIRQ(INTADASGL_IRQn);
                proc = PROC_INTERVAL_WAIT;
            }
            break;
        /*--- Stop ---*/
        case PROC_STOP:
            {
                if (adc_ex_stop(&instance.adc_ex) != TXZ_SUCCESS)
                {
                    /* no processing */
                }
                timer_stop(&instance.timer);
                /*----------------------*/
                /* Interrupt Disable    */
                /*----------------------*/
                NVIC_DisableIRQ(INTADASGL_IRQn);
                proc = PROC_ERROR;
            }
            break;
        /*--- Interval Wait ---*/
        case PROC_INTERVAL_WAIT:
            {
                if (info.update == TXZ_ENABLE)
                {
                    /*>>> Critical information. */
                    info.update = TXZ_DISABLE;
                    /*<<< Critical information. */
                    /* Blocking Conversion Start */
                    state = CONVERSION_RUN;
                    if (adc_ex_start(&instance.adc_ex) == TXZ_SUCCESS)
                    {
                        proc = PROC_CONVERSION;
                    }
                    else
                    {
                        proc = PROC_STOP;
                    }
                }
            }
            break;
        /*--- Conversion ---*/
        case PROC_CONVERSION:
            {
                /* Conversion Wait */
                if (state != CONVERSION_RUN)
                {
                    proc = PROC_OUTPUT;
                }
            }
            break;
        /*--- Output ---*/
        case PROC_OUTPUT:
            {
                BSPPotentiometers i;

                /* Get conversion value. */
                /* Get conversion value. */
                for (i=BSP_POTENTIOMETERS_0; i<BSP_POTENTIOMETERS_MAX; i++)
                {
                    if (adc_ex_channel_get_value(&instance.adc_ex, bsp_get_potentiometers_ch(i), &data[i]) == TXZ_SUCCESS)
                    {
                        /* Output information. */
                        if (printf("Convert Result Value[VR%d]:0x%x\n", (i+1), data[i]) < 0)
                        {
                            proc = PROC_STOP;
                        }
                    }
                    else
                    {
                        proc = PROC_STOP;
                    }
                }
                proc = PROC_INTERVAL_WAIT;
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if (driver_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef DEBUG
    assert_failed(MAIN_NULL, 0);
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  Single Conversion Finish handler
  * @param  potentiometers
  * @retval -
  * @note   For Continuous Conversion
  */
/*--------------------------------------------------*/
void irq_potentiometers_sgl(BSPPotentiometers potentiometers)
{
    switch (potentiometers)
    {
    case BSP_POTENTIOMETERS_0:
    case BSP_POTENTIOMETERS_1:
        adc_ex_single_irq_handler(&instance.adc_ex);
        break;
    default:
        break;
    }   
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __nop();
    }
}
#endif
/**
 *  @}
 */ /* End of group ADC_UART_Exported_functions */

/**
 *  @}
 */ /* End of group ADC_UART */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
