/** 
 *******************************************************************************
 * @file    bsp.c
 * @brief   Board Support Package. \n
 *          TMPM4K4 Evaluation Board.
 * @version V1.0.0.2
 * $Date:: 2018-07-27 15:01:05 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "bsp.h"

#if defined(__BSP_H)
/**
 *  @addtogroup BSP
 *  @{
 */

/** 
 *  @defgroup TMPM4K4A_EVAL TMPM4K4A Evaluation Board
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Private_macro TMPM4K4A Evaluation Board Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Private_define TMPM4K4A Evaluation Board Private Define
 *  @{
 */

/** 
 *  @defgroup ELOSCConfiguration ELOSC Configuration
 *  @brief    ELOSC Configuration.
 *  @{
 */
#define ELOSC_CFG_WARM_UP_TIME  ((uint32_t)(5000))      /*!< Warm up time(us). */
#define ELOSC_CFG_CLOCK         ((uint32_t)(32768))     /*!< Clock(hz).        */
/** 
 *  @}
 */ /* End of group ELOSCConfiguration */

/** 
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Private_define TMPM4K4A Evaluation Board Private Define
 *  @{
 */

#define BSP_NULL        ((void *)0) /*!< Null Pointer */

/** 
 *  @defgroup  CGWUPLCR  CGWUPLCR Register
 *  @brief     LOSC warming up Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:27 | -          |
 *  | 26:12 | WUPTL      |
 *  | 11:8  | WUPTL      |
 *  | 7:2   | -          |
 *  | 1     | WULEF      |
 *  | 0     | WULON      |
 *  @{
 */
/* WUPTL  */
#define CGWUPLCR_WUPTL_HIGH_MASK        ((uint32_t)0x07FFF000)    /*!< WUPTL   :High Bit Mask.  */
/* WULEF  */
#define CGWUPLCR_WULEF_MASK             ((uint32_t)0x00000002)    /*!< WULEF   :Mask.           */
#define CGWUPLCR_WULEF_R_DONE           ((uint32_t)0x00000000)    /*!< WULEF   :[R]  :Done      */
#define CGWUPLCR_WULEF_R_RUNNING        ((uint32_t)0x00000002)    /*!< WULEF   :[R]  :Running   */
/* WULON  */
#define CGWUPLCR_WULON_MASK             ((uint32_t)0x00000001)    /*!< WULON   :Mask.           */
#define CGWUPLCR_WULON_W_ENABLE         ((uint32_t)0x00000001)    /*!< WULON   :[W]  :Enable     */
/** 
 *  @}
 */ /* End of group CGWUPLCR */

/** 
 *  @defgroup  RLMLOSCCR  RLMLOSCCR Register
 *  @brief     LOSC Control Register.
 *  @details   Detail.
 *  | Bit   | Bit Symbol |
 *  | :---  | :---       |
 *  | 31:1  | -          |
 *  | 0     | XTEN       |
 *  @{
 */
/* XTEN */
#define RLMLOSCCR_XTEN_RW_DISABLE       ((uint32_t)0x00000000)    /*!< XTEN :[R/W]  :Disable  */
#define RLMLOSCCR_XTEN_RW_ENABLE        ((uint32_t)0x00000001)    /*!< XTEN :[R/W]  :Enable   */
/** 
 *  @}
 */ /* End of group RLMLOSCCR */

/** 
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Private_define TMPM4K4A Evaluation Board Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Private_typedef TMPM4K4A Evaluation Board Private Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief   Timer Information Table.
 * @details   Detail.
 * | [][0] | [][1] | [][2] |
 * | :---  | :---  | :---  |
 * | CH    | Type  | IRQ   |
*/
/*----------------------------------*/
typedef struct
{
    uint32_t  ch;       /*!< channel  */
    uint32_t  type;     /*!< type     */
    IRQn_Type irq;      /*!< irq Type */
} bsp_timer_info_t;

/*----------------------------------*/
/** 
 * @brief   Output Pulse Information Table.
 * @details   Detail.
 * | [][0] | [][1] | [][2] |
 * | :---  | :---  | :---  |
 * | CH    | Type  | IRQ   |
*/
/*----------------------------------*/
typedef struct
{
    uint32_t  ch;       /*!< channel  */
    uint32_t  type;     /*!< type     */
    IRQn_Type irq;      /*!< irq Type */
} bsp_output_pulse_info_t;

/**
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Private_variables TMPM4K4A Evaluation Board Private Variables
 *  @{
 */
static gpio_t port;     /*!< Port driver instance. */
static BSPExInt exi;

/*--------------------------*/
/* Name    | Pin    | Port  */
/*--------------------------*/
/* USB_TXD | 17     | PK1   */
/*--------------------------*/
/*----------------------------------*/
/** 
 * @brief   USB UART TX Information Table.
 * @details   Detail.
 * | [][0] | [][1] | [][2] | [][3]  | [][4]         | 
 * | :---  | :---  | :---  | :---   | :---          |
 * | Group | Num   | CH    | DMA CH | Baudrate(bps) |
*/
/*----------------------------------*/
static const uint32_t usbUartTxTbl[BSP_USB_UART_MAX][5] =
{
    {(uint32_t)GPIO_PORT_K, (uint32_t)GPIO_PORT_1, (uint32_t)(0), (uint32_t)(9), (uint32_t)(115200)},
    {(uint32_t)GPIO_PORT_C, (uint32_t)GPIO_PORT_1, (uint32_t)(0), (uint32_t)(15), (uint32_t)(115200)}
};

/*-----------------------------*/
/* Name        | Pin    | Port */
/*-----------------------------*/
/* USB_RXD     | 16     | PK0  */
/*-----------------------------*/
/*----------------------------------*/
/** 
 * @brief   USB UART RX Information Table.
 * @details   Detail.
 * | [][0] | [][1] | [][2] | [][3]  | [][4]         | 
 * | :---  | :---  | :---  | :---   | :---          |
 * | Group | Num   | CH    | DMA CH | Baudrate(bps) |
*/
/*----------------------------------*/
static const uint32_t usbUartRxTbl[BSP_USB_UART_MAX][5] =
{
     {(uint32_t)GPIO_PORT_K, (uint32_t)GPIO_PORT_0, (uint32_t)(0), (uint32_t)(8), (uint32_t)(115200)},
     {(uint32_t)GPIO_PORT_C, (uint32_t)GPIO_PORT_0, (uint32_t)(0), (uint32_t)(14), (uint32_t)(115200)}
};

/*----------------------------------*/
/** 
 * @brief   TSPI TX Information Table.
 * @details   Detail.
 * | [][0] | [][1] | [][2] | [][3]  | 
 * | :---  | :---  | :---  | :---   |
 * | Group | Num   | CH    | DMA CH |
*/
/*----------------------------------*/
#if defined(TMPM4K4A)
static const uint32_t tspiTxTbl[BSP_SFLASH_MAX][4] =
{
    {(uint32_t)GPIO_PORT_K, (uint32_t)GPIO_PORT_3, (uint32_t)(0), (uint32_t)(1)},
    {(uint32_t)GPIO_PORT_A, (uint32_t)GPIO_PORT_0, (uint32_t)(1), (uint32_t)(3)},
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_0, (uint32_t)(2), (uint32_t)(5)},
    {(uint32_t)GPIO_PORT_C, (uint32_t)GPIO_PORT_0, (uint32_t)(3), (uint32_t)(7)}
};
#elif defined(TMPM4K2A) || defined(TMPM4K1A)
static const uint32_t tspiTxTbl[BSP_SFLASH_MAX][4] =
{
    {(uint32_t)GPIO_PORT_K, (uint32_t)GPIO_PORT_3, (uint32_t)(0), (uint32_t)(1)},
    {(uint32_t)GPIO_PORT_A, (uint32_t)GPIO_PORT_0, (uint32_t)(1), (uint32_t)(3)},
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_0, (uint32_t)(2), (uint32_t)(5)},
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_0, (uint32_t)(2), (uint32_t)(5)}
};
#else
static const uint32_t tspiTxTbl[BSP_SFLASH_MAX][4] =
{
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_0, (uint32_t)(2), (uint32_t)(5)},
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_0, (uint32_t)(2), (uint32_t)(5)},
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_0, (uint32_t)(2), (uint32_t)(5)},
    {(uint32_t)GPIO_PORT_G, (uint32_t)GPIO_PORT_0, (uint32_t)(2), (uint32_t)(5)}
};
#endif
/*-----------------------------*/
/* Name        | Pin    | Port */
/*-----------------------------*/
/* PORT_LED[0] | 30     | PJ0  */
/* PORT_LED[1] | 31     | PJ2  */
/* PORT_LED[2] | 32     | PJ4  */
/* PORT_LED[3] | 33     | PL4  */
/*-----------------------------*/
/*----------------------------------*/
/** 
 * @brief   Port LED Information Table.
 * @details   Detail.
 * | [][0] | [][1] |
 * | :---  | :---  |
 * | Group | Num   |
*/
/*----------------------------------*/
static const uint32_t ledTbl[BSP_LED_MAX][2] =
{
    {(uint32_t)GPIO_PORT_J, (uint32_t)GPIO_PORT_0},
    {(uint32_t)GPIO_PORT_J, (uint32_t)GPIO_PORT_2},
    {(uint32_t)GPIO_PORT_J, (uint32_t)GPIO_PORT_4},
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_4}
};

/*-----------------------------*/
/* Name        | Pin    | Port */
/*-----------------------------*/
/* PORT_PSW[3] | 69     | PL0  */
/* PORT_PSW[2] | 68     | PL1  */
/* PORT_PSW[1] | 67     | PL2  */
/* PORT_PSW[0] | 66     | PL3  */
/*-----------------------------*/
/*----------------------------------*/
/** 
 * @brief   Push SW Information Table.
 * @details   Detail.
 * | [][0] | [][1] |
 * | :---  | :---  |
 * | Group | Num   |
*/
/*----------------------------------*/
static const uint32_t pswTbl[BSP_PSW_MAX][2] =
{
#if defined(DNF_CGRST_SAMPLE) || defined(GPIO_LED_SAMPLE)
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_0},
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_1},
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_2},
    {(uint32_t)GPIO_PORT_L, (uint32_t)GPIO_PORT_3}
#else
    {(uint32_t)GPIO_PORT_E, (uint32_t)GPIO_PORT_2},
    {(uint32_t)GPIO_PORT_E, (uint32_t)GPIO_PORT_3},
    {(uint32_t)GPIO_PORT_E, (uint32_t)GPIO_PORT_4},
    {(uint32_t)GPIO_PORT_E, (uint32_t)GPIO_PORT_5}
#endif
};

/*----------------------------------*/
/** 
 * @brief   Timer Information Table.
*/
/*----------------------------------*/
static const bsp_timer_info_t timerTbl[BSP_TIMER_MAX] =
{
    {(uint32_t)0, (uint32_t)T32A_TIMERA, INTT32A0AC_IRQn},
    {(uint32_t)2, (uint32_t)T32A_TIMERA, INTT32A2AC_IRQn}
};

/*----------------------------------*/
/** 
 * @brief   Output Pulse Information Table.
*/
/*----------------------------------*/
static const bsp_output_pulse_info_t pulseOutTbl[BSP_OUT_PULSE_SAMPLE_MAX] =
{
    {(uint32_t)3, (uint32_t)T32A_TIMERA, INTT32A3AC_IRQn},
    {(uint32_t)3, (uint32_t)T32A_TIMERB, INTT32A3B_IRQn}
};

/*-----------------------------*/
/* Name        | Pin    | Port */
/*-----------------------------*/
/* PORT_AIN[2] | 49     | PE5  */
/*-----------------------------*/
/*----------------------------------*/
/** 
 * @brief   Potentiometers Information Table.
 * @details   Detail.
 * | [][0] | [][1] | [][2]    | [][3]  |
 * | :---  | :---  | :---     | :---
 * | Group | Num   | ADIN CH  | ADC CH |
*/
/*----------------------------------*/
static const uint32_t potentiometersTbl[BSP_POTENTIOMETERS_MAX][4] =
{
    {(uint32_t)GPIO_PORT_E, (uint32_t)GPIO_PORT_4, (uint32_t)(14), (uint32_t)(14)},
    {(uint32_t)GPIO_PORT_E, (uint32_t)GPIO_PORT_5, (uint32_t)(15), (uint32_t)(15)},
};
/**
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Private_fuctions TMPM4K4A Evaluation Board Private Fuctions
 *  @{
 */
static void bsp_error(void);
static void fsys_all_enable(void);
static void fsys_all_disable(void);
static void dnf_clock_enable(void);
static void dnf_clock_disable(void);
static void adc_clock_enable(void);
static void adc_clock_disable(void);
static void port_initialize(void);
static void port_finalize(void);
static void ib_imc_clear_flag(uint32_t ch);
/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void bsp_error(void)
{
#ifdef BSP_DEBUG
    while(1)
    {
        __nop();
    }
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  CG Fsys All Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void fsys_all_enable(void)
{
    /*--- [CGFSYSENA] ---*/
    TSB_CG_FSYSENA_IPENA00 = 1;     /* Port A  */
    TSB_CG_FSYSENA_IPENA01 = 1;     /* Port B  */
    TSB_CG_FSYSENA_IPENA02 = 1;     /* Port C  */
    TSB_CG_FSYSENA_IPENA03 = 1;     /* Port D  */
    TSB_CG_FSYSENA_IPENA04 = 1;     /* Port E  */
    TSB_CG_FSYSENA_IPENA05 = 1;     /* Port F  */
    TSB_CG_FSYSENA_IPENA06 = 1;     /* Port G  */
    TSB_CG_FSYSENA_IPENA07 = 1;     /* Port H  */
    TSB_CG_FSYSENA_IPENA08 = 1;     /* Port J  */
    TSB_CG_FSYSENA_IPENA09 = 1;     /* Port K  */
    TSB_CG_FSYSENA_IPENA10 = 1;     /* Port L  */
    TSB_CG_FSYSENA_IPENA16 = 1;     /* TSPI 0  */
    TSB_CG_FSYSENA_IPENA17 = 1;     /* TSPI 1  */
    TSB_CG_FSYSENA_IPENA18 = 1;     /* TSPI 2  */
    TSB_CG_FSYSENA_IPENA19 = 1;     /* TSPI 3  */
    TSB_CG_FSYSENA_IPENA20 = 1;     /* UART 0 */
    TSB_CG_FSYSENA_IPENA21 = 1;     /* UART 1 */
    TSB_CG_FSYSENA_IPENA22 = 1;     /* UART 2 */
    TSB_CG_FSYSENA_IPENA23 = 1;     /* UART 3 */
    TSB_CG_FSYSENA_IPENA24 = 1;     /* I2C       */
    TSB_CG_FSYSENA_IPENA25 = 1;     /* T32A 0 */
    TSB_CG_FSYSENA_IPENA26 = 1;     /* T32A 1  */
    TSB_CG_FSYSENA_IPENA27 = 1;     /* T32A 2  */
    TSB_CG_FSYSENA_IPENA28 = 1;     /* T32A 3  */
    TSB_CG_FSYSENA_IPENA29 = 1;     /* T32A 4  */
    TSB_CG_FSYSENA_IPENA30 = 1;     /* T32A 5  */
    /*--- [CGFSYSENB] ---*/
    TSB_CG_FSYSENB_IPENB00 = 1;     /* ADC     */
    TSB_CG_FSYSENB_IPENB01 = 1;     /* Sample Hold circuit   */
    TSB_CG_FSYSENB_IPENB02 = 1;     /* AMP   */
    TSB_CG_FSYSENB_IPENB07 = 1;     /* A-ENC  */
    TSB_CG_FSYSENB_IPENB08 = 1;     /* PMD 0  */
    TSB_CG_FSYSENB_IPENB09 = 1;     /* PMD 1  */
    TSB_CG_FSYSENB_IPENB10 = 1;     /* A-VE+  */
    TSB_CG_FSYSENB_IPENB11 = 1;     /* RAM Parity  */
    TSB_CG_FSYSENB_IPENB12 = 1;     /* CRC  */
    TSB_CG_FSYSENB_IPENB13 = 1;     /* OFD  */
    TSB_CG_FSYSENB_IPENB14 = 1;     /* TRIM  */
    TSB_CG_FSYSENB_IPENB15 = 1;     /* TRGSEL  */
    TSB_CG_FSYSENB_IPENB16 = 1;     /* DMAC  */
    TSB_CG_FSYSENB_IPENB31 = 1;     /* WDT  */
}

/*--------------------------------------------------*/
/** 
  * @brief  CG Fsys All Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void fsys_all_disable(void)
{
    /*--- [CGFSYSENA] ---*/
    TSB_CG_FSYSENA_IPENA00 = 0;     /* Port A  */
    TSB_CG_FSYSENA_IPENA01 = 0;     /* Port B  */
    TSB_CG_FSYSENA_IPENA02 = 0;     /* Port C  */
    TSB_CG_FSYSENA_IPENA03 = 0;     /* Port D  */
    TSB_CG_FSYSENA_IPENA04 = 0;     /* Port E  */
    TSB_CG_FSYSENA_IPENA05 = 0;     /* Port F  */
    TSB_CG_FSYSENA_IPENA06 = 0;     /* Port G  */
    TSB_CG_FSYSENA_IPENA07 = 0;     /* Port H  */
    TSB_CG_FSYSENA_IPENA08 = 0;     /* Port J  */
    TSB_CG_FSYSENA_IPENA09 = 0;     /* Port K  */
    TSB_CG_FSYSENA_IPENA10 = 0;     /* Port L  */
    TSB_CG_FSYSENA_IPENA16 = 0;     /* TSPI 0  */
    TSB_CG_FSYSENA_IPENA17 = 0;     /* TSPI 1  */
    TSB_CG_FSYSENA_IPENA18 = 0;     /* TSPI 2  */
    TSB_CG_FSYSENA_IPENA19 = 0;     /* TSPI 3  */
    TSB_CG_FSYSENA_IPENA20 = 0;     /* UART 0 */
    TSB_CG_FSYSENA_IPENA21 = 0;     /* UART 1 */
    TSB_CG_FSYSENA_IPENA22 = 0;     /* UART 2 */
    TSB_CG_FSYSENA_IPENA23 = 0;     /* UART 3 */
    TSB_CG_FSYSENA_IPENA24 = 0;     /* I2C       */
    TSB_CG_FSYSENA_IPENA25 = 0;     /* T32A 0 */
    TSB_CG_FSYSENA_IPENA26 = 0;     /* T32A 1  */
    TSB_CG_FSYSENA_IPENA27 = 0;     /* T32A 2  */
    TSB_CG_FSYSENA_IPENA28 = 0;     /* T32A 3  */
    TSB_CG_FSYSENA_IPENA29 = 0;     /* T32A 4  */
    TSB_CG_FSYSENA_IPENA30 = 0;     /* T32A 5  */
    /*--- [CGFSYSENB] ---*/
    TSB_CG_FSYSENB_IPENB00 = 0;     /* ADC     */
    TSB_CG_FSYSENB_IPENB01 = 0;     /* Sample Hold circuit   */
    TSB_CG_FSYSENB_IPENB02 = 0;     /* AMP   */
    TSB_CG_FSYSENB_IPENB07 = 0;     /* A-ENC  */
    TSB_CG_FSYSENB_IPENB08 = 0;     /* PMD 0  */
    TSB_CG_FSYSENB_IPENB09 = 0;     /* PMD 1  */
    TSB_CG_FSYSENB_IPENB10 = 0;     /* A-VE+  */
    TSB_CG_FSYSENB_IPENB11 = 0;     /* RAM Parity  */
    TSB_CG_FSYSENB_IPENB12 = 0;     /* CRC  */
    TSB_CG_FSYSENB_IPENB13 = 0;     /* OFD  */
    TSB_CG_FSYSENB_IPENB14 = 0;     /* TRIM  */
    TSB_CG_FSYSENB_IPENB15 = 0;     /* TRGSEL  */
    TSB_CG_FSYSENB_IPENB16 = 0;     /* DMAC  */
    TSB_CG_FSYSENB_IPENB31 = 0;     /* WDT  */
}

/*--------------------------------------------------*/
/** 
  * @brief  DNF Clock Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void dnf_clock_enable(void)
{
    /*--- [CGFCEN:DNFCKEN] ---*/
    
    TSB_CG_FCEN_DNFCKEN  = 1;       /* DNF CLOCK ENABLE */
}

/*--------------------------------------------------*/
/** 
  * @brief  DNF Clock Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void dnf_clock_disable(void)
{
    /*--- [CGFCEN:DNFCKEN] ---*/
    TSB_CG_FCEN_DNFCKEN  = 0;       /* DNF CLOCK DISABLE */
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC Clock Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void adc_clock_enable(void)
{
    /*--- [CGSPCLKEN:ADCKEN] ---*/
    TSB_CG_SPCLKEN_ADCKEN = 1;
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC Clock Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void adc_clock_disable(void)
{
    /*--- [CGSPCLKEN:ADCKEN] ---*/
    TSB_CG_SPCLKEN_ADCKEN = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  Port Initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void port_initialize(void)
{
#if defined(TMPM4K4A) || defined(TMPM4K2A) || defined(TMPM4K1A)
    /*===============================================================================*/
    /*  [Port A]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PADATA | PACR | PAFRn | PAOD | PAPUP | PADN | PAIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PA0  | TSPI1TXD  | O   | 0      | 1    | PAFR3 | 0    | 0     | 0    | 0     */
    /*  PA1  | TSPI1RXD  | I   | 0      | 0    | PAFR3 | 0    | 0     | 0    | 1     */
    /*       | /INT09    | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PA2  | TSPI1CLK  | O   | 0      | 1    | PAFR3 | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pa_instance = TSB_PA;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_A) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
#ifdef DNF_CGRST_SAMPLE
    /*- PA0 -*/
    /* Use reset value.  */
    /*- PA1 -*/
    if (gpio_func(&port, GPIO_PORT_A, 1, GPIO_PA1_INT09, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PA2 -*/
    /* Use reset value.  */
#else
    /*- PA0 -*/
    if (gpio_func(&port, GPIO_PORT_A, 0, GPIO_PA0_TSPI1TXD, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PA1 -*/
    if (gpio_func(&port, GPIO_PORT_A, 1, GPIO_PA1_TSPI1RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PA2 -*/
    if (gpio_func(&port, GPIO_PORT_A, 2, GPIO_PA2_TSPI1CLK, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
#endif
    /*===============================================================================*/
    /*  [Port B]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PBDATA | PBCR | PBFRn | PBOD | PBPUP | PBDN | PBIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PB0  | ISC0SDA   | I/O | 0      | 1    | PCFR3 | 1    | 0     | 0    | 1     */
    /*  PB1  | I2CSCL    | I/O | 0      | 1    | PCFR3 | 1    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pb_instance = TSB_PB;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_B) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PB0 -*/
    if (gpio_func(&port, GPIO_PORT_B, 0, GPIO_PB0_I2C0SDA, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PB1 -*/
    if (gpio_func(&port, GPIO_PORT_B, 1, GPIO_PB1_I2CSCL, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /* PBOD Setting */
    if (gpio_write_mode(&port, GPIO_PORT_B, GPIO_Mode_OD, 0x03) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*===============================================================================*/
    /*  [Port C]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PCDATA | PCCR | PCFRn | PCOD | PCPUP | PCDN | PCIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PC0  |T32A03OUTA | O   | 0      | 1    | PCFR4 | 0    | 0     | 0    | 0     */
    /*  PC1  |TSPI3RXD   | I   | 0      | 0    | PCFR3 | 0    | 0     | 0    | 1     */
    /*  PC2  |TSPI3CLK   | I   | 0      | 0    | PCFR3 | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pc_instance = TSB_PC;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_C) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PC0 -*/
    if (gpio_func(&port, GPIO_PORT_C, 0, GPIO_PC0_T32A03OUTA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PC1 -*/
    if (gpio_func(&port, GPIO_PORT_C, 1, GPIO_PC1_TSPI3RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PC2 -*/
    if (gpio_func(&port, GPIO_PORT_C, 2, GPIO_PC2_TSPI3CLK, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
#endif /* defined(TMPM4K4A) || defined(TMPM4K2A) || defined(TMPM4K1A) */
    /*===============================================================================*/
    /*  [Port D]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PDDATA | PDCR | PDFRn | PDOD | PDPUP | PDDN | PDIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PD0  | AINA00    | -   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PD1  | AINA01/02 | -   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PD2  | AINA03    | -   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PD3  | AINA04/05 | -   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PD4  | AINA06    | -   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PD5  | AINA07/08 | -   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PD6  | AINA09    | -   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pd_instance = TSB_PD;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_D) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PD0 -*/
    /* Use reset value.  */
    /*- PD1 -*/
    /* Use reset value.  */
    /*- PD2 -*/
    /* Use reset value.  */
    /*- PD3 -*/
    /* Use reset value.  */
    /*==================================================================================*/
    /*  [Port E]                                                                        */
    /*----------------------------------------------------------------------------------*/
    /*  Port | Function     | I/O | PEDATA | PECR | PEFRn | PEOD | PEPUP | PEDN | PEIE  */
    /*----------------------------------------------------------------------------------*/
    /*  PE0  | AINA10       | -   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PE1  | AINA11       | -   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PE2  | INPUT        | I   | 0      | 0    | N/A   | 0    | 0     | 0    | 1     */
    /*  PE3  | INPUT        | I   | 0      | 0    | N/A   | 0    | 0     | 0    | 1     */
    /*  PE4  | AINA14       | I   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*  PE5  | VREFH/AINA15 | I   | 0      | 0    | N/A   | 0    | 0     | 0    | 0     */
    /*==================================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pe_instance = TSB_PE;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_E) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PE0 -*/
    /* Use reset value.  */
    /*- PE1 -*/
    /* Use reset value.  */
    /*- PE2 -*/
    if (gpio_write_bit(&port, GPIO_PORT_E, 2, GPIO_Mode_IE, GPIO_PIN_SET) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PE3 -*/
    if (gpio_write_bit(&port, GPIO_PORT_E, 3, GPIO_Mode_IE, GPIO_PIN_SET) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PE4 -*/
    /* Use reset value.  */
    /*- PE5 -*/
    /* Use reset value.  */
#if defined(TMPM4K4A) || defined(TMPM4K2A) || defined(TMPM4K1A)
    /*===============================================================================*/
    /*  [Port F]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PFDATA | PFCR | PFFRn | PFOD | PFPUP | PFDN | PFIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PF0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PF1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  TPF2 | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pf_instance = TSB_PF;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_F) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PF0 -*/
    /* Use reset value.  */
    /*- PF1 -*/
    /* Use reset value.  */
    /*- TPF2 -*/
    /* Use reset value.  */
#endif /* defined(TMPM4K4A) || defined(TMPM4K2A) || defined(TMPM4K1A) */
    /*===============================================================================*/
    /*  [Port G]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PGDATA | PGCR | PGFRn | PGOD | PGPUP | PGDN | PGIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PG0  | T32A2OUTA | O   | 0      | 1    | PGFR3 | 0    | 0     | 0    | 0     */
    /*  PG1  | TSPI2RXD/ | I   | 0      | 0    | PGFR2 | 0    | 0     | 0    | 1     */
    /*       | T32A2INA0 | I   | 0      | 0    | PGFR3 | 0    | 0     | 0    | 1     */
    /*  PG2  | TSPI2CLK  | I   | 0      | 0    | PGFR2 | 0    | 0     | 0    | 1     */
    /*  PG3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pg_instance = TSB_PG;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_G) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PG0 -*/
    if (gpio_func(&port, GPIO_PORT_G, 0, GPIO_PG0_T32A2OUTA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PG1 -*/
#ifdef TRMOSC_SAMPLE
    /*- PG1 -*/
    if (gpio_func(&port, GPIO_PORT_G, 1, GPIO_PG1_T32A2INA0, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }    
#else
    /*- PG1 -*/
    if (gpio_func(&port, GPIO_PORT_G, 1, GPIO_PG1_TSPI2RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PG2 -*/
    if (gpio_func(&port, GPIO_PORT_G, 2, GPIO_PG2_TSPI2CLK, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
#endif
    /*- PG3 -*/
    /* Use reset value.  */
    /*- PG4 -*/
    /* Use reset value.  */
    /*- PG5 -*/
    /* Use reset value.  */

    /*===============================================================================*/
    /*  [Port H]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PHDATA | PHCR | PHFRn | PHOD | PHPUP | PHDN | PHIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PH0  | X1        | I   | 0      | N/A  | N/A   | N/A  | N/A   | 0    | 0     */
    /*  PH1  | X2        | O   | 0      | N/A  | N/A   | N/A  | N/A   | 0    | 0     */
    /*  PH2  | -         | I   | 0      | N/A  | N/A   | N/A  | N/A   | 0    | 0     */
    /*  PH3  | -         | O   | 0      | N/A  | N/A   | N/A  | N/A   | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_ph_instance = TSB_PH;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_H) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PH0 -*/
    if (gpio_func(&port, GPIO_PORT_H, 0, GPIO_PH0_X1, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PH1 -*/
    if (gpio_func(&port, GPIO_PORT_H, 1, GPIO_PH1_X2, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PH2 -*/
    /* Use reset value.  */
    /*- PH3 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port J]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PJDATA | PJCR | PJFRn | PJOD | PJPUP | PJDN | PJIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PJ0  | OUTPUT    | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ2  | OUTPUT    | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ4  | OUTPUT    | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PJ7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pj_instance = TSB_PJ;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_J) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PJ0 -*/
    if (gpio_func(&port, GPIO_PORT_J, 0, GPIO_PJ0_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PJ1 -*/
    /* Use reset value.  */
    /*- PJ2 -*/
    if (gpio_func(&port, GPIO_PORT_J, 2, GPIO_PJ2_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PJ3 -*/
    /* Use reset value.  */
    /*- PJ4 -*/
    if (gpio_func(&port, GPIO_PORT_J, 4, GPIO_PJ4_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PJ5 -*/
    /* Use reset value.  */
    /*- PJ6 -*/
    /* Use reset value.  */
    /*- PJ7 -*/
    /* Use reset value.  */
    /*===============================================================================*/
    /*  [Port K]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PKDATA | PKCR | PKFRn | PKOD | PKPUP | PKDN | PKIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PK0  | UT0RXD    | -   | 0      | 0    | PKFR1 | 0    | 1     | 0    | 1     */
    /*  PK1  | UT0TXDA   | -   | 0      | 1    | PKFR1 | 0    | 0     | 0    | 0     */
    /*  PK2  | TMS/SWDIO | -   | 0      | 1    | PKFR7 | 0    | 1     | 0    | 1     */
    /*  PK3  | TCK/SWCLK | -   | 0      | 0    | PKFR7 | 0    | 0     | 1    | 1     */
    /*  PK4  | TRST_N    | -   | 0      | 0    | PKFR7 | 0    | 1     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pk_instance = TSB_PK;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_K) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PK0 -*/
    if (gpio_func(&port, GPIO_PORT_K, 0, GPIO_PK0_UT0RXD, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PK1 -*/
    if (gpio_func(&port, GPIO_PORT_K, 1, GPIO_PK1_UT0TXDA, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PK2 -*/
    /* Use reset value.  */
    /*- PK3 -*/
    /* Use reset value.  */
    /*- PK4 -*/
    /* Use reset value.  */
#if defined(TMPM4K4A)
    /*===============================================================================*/
    /*  [Port L]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PLDATA | PLCR | PLFRn | PLOD | PLPUP | PLDN | PLIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PL0  | INPUT     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PL1  | INPUT     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PL2  | INPUT     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PL3  | INPUT     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PL4  | OUTPUT    | O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PL5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pl_instance = TSB_PL;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_L) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Setting    ---*/
    /*- PL0 -*/
    if (gpio_func(&port, GPIO_PORT_L, 0, GPIO_PL0_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PL1 -*/
    if (gpio_func(&port, GPIO_PORT_L, 1, GPIO_PL1_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PL2 -*/
    if (gpio_func(&port, GPIO_PORT_L, 2, GPIO_PL2_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PL3 -*/
    if (gpio_func(&port, GPIO_PORT_L, 3, GPIO_PL3_INPUT, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PL4 -*/
    if (gpio_func(&port, GPIO_PORT_L, 4, GPIO_PL4_OUTPUT, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*- PL5 -*/
    /* Use reset value.  */
    /*- PL6 -*/
    /* Use reset value.  */
#endif /* defined(TMPM4K4A) */
}

/*--------------------------------------------------*/
/** 
  * @brief  Port Finalize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void port_finalize(void)
{
#if defined(TMPM4K4A) || defined(TMPM4K2A) || defined(TMPM4K1A)
    /*======================*/
    /*  [Port A]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_A) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pa_instance = BSP_NULL;
    /*======================*/
    /*  [Port B]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_B) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pb_instance = BSP_NULL;
    /*======================*/
    /*  [Port C]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_C) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pc_instance = BSP_NULL;
#endif /* defined(TMPM4K4A) || defined(TMPM4K2A) || defined(TMPM4K1A) */
    /*======================*/
    /*  [Port D]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_D) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pd_instance = BSP_NULL;
    /*======================*/
    /*  [Port E]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_E) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pe_instance = BSP_NULL;
#if defined(TMPM4K4A) || defined(TMPM4K2A) || defined(TMPM4K1A)
    /*======================*/
    /*  [Port F]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_F) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pf_instance = BSP_NULL;
#endif /* defined(TMPM4K4A) || defined(TMPM4K2A) || defined(TMPM4K1A) */
    /*======================*/
    /*  [Port G]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_G) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pg_instance = BSP_NULL;
    /*======================*/
    /*  [Port H]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_H) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_ph_instance = BSP_NULL;
    /*======================*/
    /*  [Port J]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_J) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pj_instance = BSP_NULL;
    /*======================*/
    /*  [Port K]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_K) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pk_instance = BSP_NULL;
#if defined(TMPM4K4A)
    /*======================*/
    /*  [Port L]            */
    /*======================*/
    /*--- Finalize ---*/
    if (gpio_deinit(&port, GPIO_PORT_L) != TXZ_SUCCESS)
    {
        bsp_error();
    }
    /*--- Destruct ---*/
    /* Register Release */
    port.p_pl_instance = BSP_NULL;
#endif /* defined(TMPM4K4A) */
}

/*--------------------------------------------------*/
/** 
  * @brief  IBIMC clear flag.
  * @param  ch :Channel No.
  * @return -
  * @retval -
  * @note   When INT enable.
  */
/*--------------------------------------------------*/
static void ib_imc_clear_flag(uint32_t ch)
{
    __IO uint8_t *p_imc = (__IO uint8_t *)(&TSB_IB->IMC000 + (ch));

    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 1      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-0 | reserved     | -      */
    /*-----------------------------*/
    *p_imc = (uint8_t)0xC0;
    /* Dummy read is need. */
    {
        __IO uint8_t imc = *p_imc;
    }
}

/**
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TMPM4K4A_EVAL_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_initialize(void)
{
    /*--- Fsys Enable      ---*/
    fsys_all_enable();
    /*--- DNF Clock Enable ---*/
    dnf_clock_enable();
    /*--- ADC Clock Enable ---*/
    adc_clock_enable();
    /*--- Port             ---*/
    port_initialize();
}

/*--------------------------------------------------*/
/** 
  * @brief  Finalize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_finalize(void)
{
    /*--- Port              ---*/
    port_finalize();
    /*--- DNF Clock Disable ---*/
    dnf_clock_disable();
    /*--- ADC Clock Disable ---*/
    adc_clock_disable();
    /*--- Fsys Disable      ---*/
    fsys_all_disable();
}

/*--------------------------------------------------*/
/** 
  * @brief  Port Driver Instance Addess Get Function.
  * @param  -
  * @return Port Driver Instance Addess.
  * @retval != Null :Valid Value.
  * @note   -
  */
/*--------------------------------------------------*/
gpio_t *bsp_get_gpio_instance_address(void)
{
    return (&port);
}

/*--------------------------------------------------*/
/** 
  * @brief  Port LED GPIO Group Get Function.
  * @param  led     :Port LED.
  * @return Port LED GPIO Group. \n
  *         Value is @ref gpio_gr_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_gpio_group_led(BSPLed led)
{
    return (ledTbl[(uint32_t)led][0]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Port LED GPIO Number Get Function.
  * @param  led     :Port LED.
  * @return Port LED GPIO Number. \n
  *         Value is @ref gpio_num_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_gpio_num_led(BSPLed led)
{
    return (ledTbl[(uint32_t)led][1]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Push SW GPIO Group Get Function.
  * @param  psw :Push SW.
  * @return Push SW GPIO Group. \n
  *         Value is @ref gpio_gr_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_gpio_group_psw(BSPPsw psw)
{
    return (pswTbl[(uint32_t)psw][0]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Push SW GPIO Number Get Function.
  * @param  psw :Push SW.
  * @return Push SW GPIO Number. \n
  *         Value is @ref gpio_num_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_gpio_num_psw(BSPPsw psw)
{
    return (pswTbl[(uint32_t)psw][1]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Channel Get Function.
  * @param  uart :USB UART.
  * @return UART Channel.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_tx_ch(BSPUsbUart uart)
{
    return (usbUartTxTbl[(uint32_t)uart][2]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Interrupt Number Get Function.
  * @param  uart :USB UART.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_usb_uart_tx_nvic(BSPUsbUart uart)
{
    return (INTSC0TX_IRQn);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Interrupt Number Get Function.
  * @param  uart :USB UART.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Receiving
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_usb_uart_rx_nvic(BSPUsbUart uart)
{
    return (INTSC0RX_IRQn);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Interrupt Number Get Function.
  * @param  uart :USB UART.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Error
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_usb_uart_err_nvic(BSPUsbUart uart)
{
    return (INTSC0ERR_IRQn);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART DMA Channel Get Function.
  * @param  uart :USB UART.
  * @return DMA Channel.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_tx_dma_ch(BSPUsbUart uart)
{
    return (usbUartTxTbl[(uint32_t)uart][3]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART DMA Channel Get Function.
  * @param  uart :USB UART.
  * @return DMA Channel.
  * @retval -
  * @note   for Receiving
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_rx_dma_ch(BSPUsbUart uart)
{
    return (usbUartRxTbl[(uint32_t)uart][3]);
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART Baudrate Get Function.
  * @param  uart :USB UART.
  * @return Baudrate(bps).
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_usb_uart_tx_baudrate(BSPUsbUart uart)
{
    return (usbUartTxTbl[(uint32_t)uart][4]);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Channel Get Function.
  * @param  tspi :TSPI.
  * @return TSPI Channel.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
uint32_t bsp_get_tspi_tx_ch(BSPSFlash tspi)
{
    return (tspiTxTbl[(uint32_t)tspi][2]);
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI Interrupt Number Get Function.
  * @param  tspi :TSPI.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   for Transmission
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_tspi_tx_nvic(BSPSFlash tspi)
{
    IRQn_Type type;
#if defined(TMPM4K4A)
    switch(tspi){
        case BSP_SFLASH_0:
            type = INTSC0TX_IRQn;
            break;
        case BSP_SFLASH_1:
            type = INTSC1TX_IRQn;
            break;
        case BSP_SFLASH_2:
            type = INTSC2TX_IRQn;
            break;
        case BSP_SFLASH_3:
            type = INTSC3TX_IRQn;
            break;
        default:
            break;
    }
#elif defined(TMPM4K2A) || defined(TMPM4K1A)
    switch(tspi){
        case BSP_SFLASH_0:
            type = INTSC0TX_IRQn;
            break;
        case BSP_SFLASH_1:
            type = INTSC0TX_IRQn;
            break;
        case BSP_SFLASH_2:
            type = INTSC2TX_IRQn;
            break;
        case BSP_SFLASH_3:
            type = INTSC2TX_IRQn;
            break;
        default:
            break;
    }
#else
    type = INTSC2TX_IRQn;
#endif
    return (type);
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer Get Function.
  * @param  timer :Timer.
  * @param  p_ch  :Timer Channel Destination Address.
  * @param  p_type:Timer Type Destination Address. \n
  *               :Value is @ref t32_type_t.
  * @return -
  * @retval -
  * @note   You can read the 1ms Timer Channel and Type.
  */
/*--------------------------------------------------*/
void bsp_get_timer(BSPTimer timer, uint32_t *p_ch, uint32_t *p_type)
{
    *p_ch   = timerTbl[(uint32_t)timer].ch;
    *p_type = timerTbl[(uint32_t)timer].type;
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer Channel Get Function.
  * @param  timer :Timer.
  * @return Timer Channel.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_timer_ch(BSPTimer timer)
{
    return (timerTbl[(uint32_t)timer].ch);
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer Type Get Function.
  * @param  timer :Timer.
  * @return Timer Type. \n
  *         Value is @ref t32_type_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_timer_type(BSPTimer timer)
{
    return (timerTbl[(uint32_t)timer].type);
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer Interrupt Number Get Function.
  * @param  timer :Timer.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_timer_nvic(BSPTimer timer)
{
    return (timerTbl[(uint32_t)timer].irq);
}

/*--------------------------------------------------*/
/** 
  * @brief  Output Pulse Channel Get Function.
  * @param  pulse :Output Pulse.
  * @return Output Pulse Channel.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_output_pulse_ch(BSPOutputPulse pulse)
{
    return (pulseOutTbl[(uint32_t)pulse].ch);
}

/*--------------------------------------------------*/
/** 
  * @brief  Output Pulse Type Get Function.
  * @param  pulse :Output Pulse.
  * @return Output Pulse Type. \n
  *         Value is @ref t32_type_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_output_pulse_type(BSPOutputPulse pulse)
{
    return (pulseOutTbl[(uint32_t)pulse].type);
}

/*--------------------------------------------------*/
/** 
  * @brief  Output Pulse Interrupt Number Get Function.
  * @param  pulse :Output Pulse.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_output_pulse_nvic(BSPOutputPulse pulse)
{
    return (pulseOutTbl[(uint32_t)pulse].irq);
}

/*--------------------------------------------------*/
/** 
  * @brief  Potentiometers ADIN Get Function
  * @param  potentiometers :Potentiometers.
  * @return Potentiometers ADIN.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_potentiometers_adin(BSPPotentiometers potentiometers)
{
    return (potentiometersTbl[(uint32_t)potentiometers][2]);
}

/*--------------------------------------------------*/
/** 
  * @brief  Potentiometers ADC Channel Get Function
  * @param  potentiometers :Potentiometers.
  * @return Potentiometers ADC Channel.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t bsp_get_potentiometers_ch(BSPPotentiometers potentiometers)
{
    return (potentiometersTbl[(uint32_t)potentiometers][3]);
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ External Interrupt Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_exint_enable(void)
{
#if defined(TMPM4K4A) || defined(TMPM4K2A)
    /*--- IBIMC038: Enable ---*/
    /*-------------------------------*/
    /* Bit | Bit Symbol   | Value    */
    /*-------------------------------*/
    /* 7   | INTNCLR      | 0        */
    /* 6   | INTPCLR      | 0        */
    /* 5   | NTNFLG       | -        */
    /* 4   | INTPFLG      | -        */
    /* 3:1 | INTMODE[2:0] | 010      */
    /* 0   | INTEN        | 1        */
    /*-------------------------------*/
    TSB_IB->IMC038 = (uint8_t)(0x05);
#else
    /* Not support */
#endif
}
/*--------------------------------------------------*/
/** 
  * @brief  IRQ External Interrupt Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_exint_disable(void)
{
#if defined(TMPM4K4A) || defined(TMPM4K2A)
    /*--- IBIMC038: Disable ---*/
    /*-------------------------------*/
    /* Bit | Bit Symbol   | Value    */
    /*-------------------------------*/
    /* 7   | INTNCLR      | 0        */
    /* 6   | INTPCLR      | 0        */
    /* 5   | NTNFLG       | -        */
    /* 4   | INTPFLG      | -        */
    /* 3:1 | INTMODE[2:0] | 010      */
    /* 0   | INTEN        | 0        */
    /*-------------------------------*/
    TSB_IB->IMC038 = (uint8_t)(0x04);
    /*--- IBIMC038: INTxCLR ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 1      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 000    */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IB->IMC038 = (uint8_t)(0xC0);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC038;
    }
#else
    /* Not support */
#endif
}
/*--------------------------------------------------*/
/** 
  * @brief  External Interrupt Number Get Function.
  * @param  -
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_exint_nvic(void)
{
#if defined(TMPM4K4A) || defined(TMPM4K2A)
    return (INT09_IRQn);
#else
    /* Not support */
    return (0);
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ External Interrupt (DNF) Enable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_dnf_enable(void)
{
#if defined(TMPM4K4A) || defined(TMPM4K2A)
    /*--- IBIMC038: Enable ---*/
    /*-------------------------------*/
    /* Bit | Bit Symbol   | Value    */
    /*-------------------------------*/
    /* 7   | INTNCLR      | 0        */
    /* 6   | INTPCLR      | 0        */
    /* 5   | NTNFLG       | -        */
    /* 4   | INTPFLG      | -        */
    /* 3:1 | INTMODE[2:0] | 010      */
    /* 0   | INTEN        | 1        */
    /*-------------------------------*/
    TSB_IB->IMC038 = (uint8_t)(0x05);
#else
    /* Not support */
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  IRQ External Interrupt (DNF) Disable
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void bsp_irq_dnf_disable(void)
{
#if defined(TMPM4K4A) || defined(TMPM4K2A)
    /*--- IBIMC038: Disable ---*/
    /*-------------------------------*/
    /* Bit | Bit Symbol   | Value    */
    /*-------------------------------*/
    /* 7   | INTNCLR      | 0        */
    /* 6   | INTPCLR      | 0        */
    /* 5   | NTNFLG       | -        */
    /* 4   | INTPFLG      | -        */
    /* 3:1 | INTMODE[2:0] | 010      */
    /* 0   | INTEN        | 0        */
    /*-------------------------------*/
    TSB_IB->IMC038 = (uint8_t)(0x04);
    /*--- IBIMC038: INTxCLR ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 1      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 000    */
    /* 0   | INTEN        | 0      */
    /*-----------------------------*/
    TSB_IB->IMC038 = (uint8_t)(0xC0);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC038;
    }
#else
    /* Not support */
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  DNF Register Get Function.
  * @param  dnf :DNF.
  * @return DNF Register.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
TSB_DNF_TypeDef * bsp_get_dnf_reg(BSPDnf dnf)
{
    return (TSB_DNFA);
}
/*--------------------------------------------------*/
/** 
  * @brief  DNF InterruptNumber Get Function.
  * @param  dnf :DNF.
  * @return InterruptNumber Noise Filter Enable Value. \n
  *         Value is @ref t32_type_t.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
#define REG_DNF_NFENCR_NFEN9_ENABLE         ((uint32_t)0x00000200)        /*!< INT9 Noise Filter Enable.    */
uint32_t bsp_get_dnf_int_num(BSPDnf dnf)
{
#if defined(TMPM4K4A) || defined(TMPM4K2A)
    return(REG_DNF_NFENCR_NFEN9_ENABLE);
#else
    /* Not support */
    return(0);
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  DNF Interrupt Number Get Function.
  * @param  dnf :DNF.
  * @return Interrupt Number. \n
  *         Value is @ref IRQn_Type.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
IRQn_Type bsp_get_dnf_nvic(BSPDnf dnf)
{
#if defined(TMPM4K4A) || defined(TMPM4K2A)
    return (INT09_IRQn);
#else
    /* Not support */
    return (0);
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  SysTick Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void SysTick_Handler(void)
{
    irq_systick();
}

__weak void irq_systick(void){
	
}
/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer done IRQ.
  * @param  ch :DMA Channel.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
__weak void irq_dmac_done(uint32_t ch)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  External Interrupt(INT01) IRQ.
  * @param  exi : External Interrupt.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
__weak void irq_exint(BSPExInt exi_p)
{
    /* Sample Soft don't use this function. */
}


/*--------------------------------------------------*/
/** 
  * @brief  INT00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   0: Interrupt Pin0
  */
/*--------------------------------------------------*/
void INT00_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT01 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   1: Interrupt Pin1
  */
/*--------------------------------------------------*/
void INT01_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT02 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   2: Interrupt Pin2
  */
/*--------------------------------------------------*/
void INT02_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT03 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   3: Interrupt Pin3
  */
/*--------------------------------------------------*/
void INT03_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT04 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   4: Interrupt Pin4
  */
/*--------------------------------------------------*/
void INT04_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT05 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   5: Interrupt Pin5
  */
/*--------------------------------------------------*/
void INT05_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT06 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   6: Interrupt Pin6
  */
/*--------------------------------------------------*/
void INT06_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT07 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   7: Interrupt Pin7
  */
/*--------------------------------------------------*/
void INT07_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INT08 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   8: Interrupt Pin8
  */
/*--------------------------------------------------*/
void INT08_IRQHandler(void)
{
    /* no process */
}

#if defined(TMPM4K4A) || defined(TMPM4K2A)
/*--------------------------------------------------*/
/** 
  * @brief  INT09 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   9: Interrupt Pin9
  */
/*--------------------------------------------------*/
void INT09_IRQHandler(void)
{
    /*--- IBIMC038: INTxCLR ---*/
    /*-----------------------------*/
    /* Bit | Bit Symbol   | Value  */
    /*-----------------------------*/
    /* 7   | INTNCLR      | 1      */
    /* 6   | INTPCLR      | 1      */
    /* 5   | INTNFLG      | 0      */
    /* 4   | INTPFLG      | 0      */
    /* 3-1 | INTMODE[2:0] | 010    */
    /* 0   | INTEN        | 1      */
    /*-----------------------------*/
    TSB_IB->IMC038 = (uint8_t)(0xC5);
    /* Dummy read is need. */
    {
        __IO uint8_t imc = TSB_IB->IMC038;
    }
    if(exi==BSP_EXINT_1){ exi =BSP_EXINT_0; }
    else { exi =BSP_EXINT_1; }
    /*--- Handler Call     ---*/
    irq_exint(exi);
}
#endif

/*--------------------------------------------------*/
/** 
  * @brief  INTVCN0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   11: A-VE interrupt 0
  */
/*--------------------------------------------------*/
void INTVCN0_IRQHandlerr(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTVCT0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   12: A-VE task termination interrupt 0
  */
/*--------------------------------------------------*/
void INTVCT0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTEMG0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   13: PMD0 EMG interrupt
  */
/*--------------------------------------------------*/
void INTEMG0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTEMG1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   14: PMD1 EMG interrupt
  */
/*--------------------------------------------------*/
void INTEMG1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTOVV0 IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   15: PMD0 OVV interrupt
  */
/*--------------------------------------------------*/
void INTOVV0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTOVV1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   16: PMD1 OVV interrupt
  */
/*--------------------------------------------------*/
void INTOVV1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTPWM0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17: PMD0 PWM interrupt
  */
/*--------------------------------------------------*/
void INTPWM0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTPWM1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   18: PMD1 PWM interrupt
  */
/*--------------------------------------------------*/
void INTPWM1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTENC00 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   19: Encoder 0 interrupt 0
  */
/*--------------------------------------------------*/
void INTENC00_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTENC01 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   20: Encoder 0 interrupt 1
  */
/*--------------------------------------------------*/
void INTENC01_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADAPDA IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   21: ADC conversion triggered by PMD is finished A
  */
/*--------------------------------------------------*/
void INTADAPDA_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADAPDB IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   22: ADC conversion triggered by PMD is finished B
  */
/*--------------------------------------------------*/
void INTADAPDB_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADAPDC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   23: ADC conversion triggered by PMD is finished C
  */
/*--------------------------------------------------*/
void INTADAPDC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADAPDD IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   24: ADC conversion triggered by PMD is finished D
  */
/*--------------------------------------------------*/
void INTADAPDD_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADAPFLG IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   25: ADC Priority interrupt
  */
/*--------------------------------------------------*/
void INTADAPFLG_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADACP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   26: ADC conversion monitoring function interrupt A
  */
/*--------------------------------------------------*/
void INTADACP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADACP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   27: ADC conversion monitoring function interrupt B
  */
/*--------------------------------------------------*/
void INTADACP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADATRG IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   28: ADC conversion triggered by General purpose is finished
  */
/*--------------------------------------------------*/
void INTADATRG_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADASGL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   29: ADC conversion triggered by Single program is finished
  */
/*--------------------------------------------------*/
void INTADASGL_IRQHandler(void)
{
    irq_potentiometers_sgl(BSP_POTENTIOMETERS_1);
}

__weak void irq_potentiometers_sgl(BSPPotentiometers potentiometers){
	
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADACNT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   30: ADC conversion triggered by Continuity program is finished
  */
/*--------------------------------------------------*/
void INTADACNT_IRQHandler(void)
{
    irq_potentiometers_cnt(BSP_POTENTIOMETERS_1);
}

__weak void irq_potentiometers_cnt(BSPPotentiometers potentiometers){
	
}
/*--------------------------------------------------*/
/** 
  * @brief  INTSC0RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   31: TSPI0 / UART0 reception interrupt
  */
/*--------------------------------------------------*/
void INTSC0RX_IRQHandler(void)
{
    irq_usb_uart_rx(BSP_USB_UART_0);
}

__weak void irq_usb_uart_rx(BSPUsbUart uart){
	
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC0TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   32: TSPI0 / UART0 transmit interrupt
  */
/*--------------------------------------------------*/
void INTSC0TX_IRQHandler(void)
{
    irq_usb_uart_tx(BSP_USB_UART_0);
}

__weak void irq_usb_uart_tx(BSPUsbUart uart){
	
}
/*--------------------------------------------------*/
/** 
  * @brief  INTSC0ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   33: TSPI0 / UART0 error interrupt
  */
/*--------------------------------------------------*/
void INTSC0ERR_IRQHandler(void)
{
    irq_usb_uart_err(BSP_USB_UART_0);
}

__weak void irq_usb_uart_err(BSPUsbUart uart){
	
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC1RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   34: TSPI1 / UART1 reception interrupt
  */
/*--------------------------------------------------*/
void INTSC1RX_IRQHandler(void)
{
    irq_sflash_rx(BSP_SFLASH_1);
}

__weak void irq_sflash_rx(BSPSFlash sflash){
	
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC1TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   35: TSPI1 / UART1 transmit interrupt
  */
/*--------------------------------------------------*/
void INTSC1TX_IRQHandler(void)
{
   irq_sflash_tx(BSP_SFLASH_1);
}

__weak void irq_sflash_tx(BSPSFlash sflash){
	
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC1ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   36: TSPI1 / UART1 error interrupt
  */
/*--------------------------------------------------*/
void INTSC1ERR_IRQHandler(void)
{
    irq_sflash_err(BSP_SFLASH_1);
}

__weak void irq_sflash_err(BSPSFlash sflash){
	
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC2RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   37: TSPI2 / UART2 reception interrupt
  */
/*--------------------------------------------------*/
void INTSC2RX_IRQHandler(void)
{
    irq_sflash_rx(BSP_SFLASH_2);
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC2TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   38: TSPI2 / UART2 transmit interrupt
  */
/*--------------------------------------------------*/
void INTSC2TX_IRQHandler(void)
{
   irq_sflash_tx(BSP_SFLASH_2);
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC2ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   39: TSPI2 / UART2 error interrupt
  */
/*--------------------------------------------------*/
void INTSC2ERR_IRQHandler(void)
{
    irq_sflash_err(BSP_SFLASH_2);
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC3RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   40: TSPI3 / UART3 reception interrupt
  */
/*--------------------------------------------------*/
void INTSC3RX_IRQHandler(void)
{
    irq_sflash_rx(BSP_SFLASH_3);
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC3TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   41: TSPI3 / UART3 transmit interrupt
  */
/*--------------------------------------------------*/
void INTSC3TX_IRQHandler(void)
{
   irq_sflash_tx(BSP_SFLASH_3);
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC3ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   42: TSPI3 / UART3 error interrupt
  */
/*--------------------------------------------------*/
void INTSC3ERR_IRQHandler(void)
{
    irq_sflash_err(BSP_SFLASH_3);
}

/*--------------------------------------------------*/
/** 
  * @brief  INTI2C0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   43: I2C0 transmission and reception interrupt
  */
/*--------------------------------------------------*/
void INTI2C0_IRQHandler(void)
{
    irq_i2c();
}

__weak void irq_i2c(void){
	
}

/*--------------------------------------------------*/
/** 
  * @brief  INTI2C0AL IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   44: I2C0 arbitration lost interrupt
  */
/*--------------------------------------------------*/
void INTI2C0AL_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTI2C0BF IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   45: I2C0 bus free interrupt
  */
/*--------------------------------------------------*/
void INTI2C0BF_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTI2C0NA IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   46: I2C0 no ack interrupt
  */
/*--------------------------------------------------*/
void INTI2C0NA_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A0AC IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   47: T32A0 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A0AC_IRQHandler(void)
{
    irq_timer(BSP_TIMER_1MS);
}

__weak void irq_timer(BSPTimer timer){
	
}
/*--------------------------------------------------*/
/** 
  * @brief  INTT32A0ACCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   48: T32A0 TimmerA/C input capture 0
  */
/*--------------------------------------------------*/
void INTT32A0ACCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A0ACCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   49: T32A0 TimmerA/C input capture 1
  */
/*--------------------------------------------------*/
void INTT32A0ACCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A0B IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   50: T32A0 TimmerB compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A0B_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A0BCAP0 IRQ Handler 
  * @param  -
  * @return -
  * @retval -
  * @note   51: T32A0 TimmerB input capture 0
  */
/*--------------------------------------------------*/
void INTT32A0BCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A0BCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   52: T32A0 TimmerB input capture 1
  */
/*--------------------------------------------------*/
void INTT32A0BCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A1AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   53: T32A1 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A1AC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A1ACCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   54: T32A1 TimmerA/C input capture 0
  */
/*--------------------------------------------------*/
void INTT32A1ACCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A1ACCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   55: T32A1 TimmerA/C input capture 1
  */
/*--------------------------------------------------*/
void INTT32A1ACCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A1B IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   56: T32A1 TimmerB compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A1B_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A1BCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   57: T32A1 TimmerB input capture 0
  */
/*--------------------------------------------------*/
void INTT32A1BCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A1BCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   58: T32A1 TimmerB input capture 1
  */
/*--------------------------------------------------*/
void INTT32A1BCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A2AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   59: T32A2 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A2AC_IRQHandler(void)
{
    irq_timer(BSP_TIMER_1S);
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A2ACCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   60: T32A2 TimmerA/C input capture 0
  */
/*--------------------------------------------------*/
void INTT32A2ACCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A2ACCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   61: T32A2 TimmerA/C input capture 1
  */
/*--------------------------------------------------*/
void INTT32A2ACCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A2B IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   62: T32A2 TimmerB compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A2B_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A2BCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   63: T32A2 TimmerB input capture 0
  */
/*--------------------------------------------------*/
void INTT32A2BCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A2BCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   64: T32A2 TimmerB input capture 1
  */
/*--------------------------------------------------*/
void INTT32A2BCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A3AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   65: T32A3 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A3AC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A3ACCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   66: T32A3 TimmerA/C input capture 0
  */
/*--------------------------------------------------*/
void INTT32A3ACCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A3ACCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   67: T32A3 TimmerA/C input capture 1
  */
/*--------------------------------------------------*/
void INTT32A3ACCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A3B IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   68: T32A3 TimmerB compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A3B_IRQHandler(void)
{
    irq_counter(BSP_COUNTER_SAMPLE_B);
}

__weak void irq_counter(BSPCounter cnt){
	
}
/*--------------------------------------------------*/
/** 
  * @brief  INTT32A3BCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   69: T32A3 TimmerB input capture 0
  */
/*--------------------------------------------------*/
void INTT32A3BCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A3BCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   70: T32A3 TimmerB input capture 1
  */
/*--------------------------------------------------*/
void INTT32A3BCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A4AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   71: T32A4 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A4AC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A4ACCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   72: T32A4 TimmerA/C input capture 0
  */
/*--------------------------------------------------*/
void INTT32A4ACCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A4ACCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   73: T32A4 TimmerA/C input capture 1
  */
/*--------------------------------------------------*/
void INTT32A4ACCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A4B IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   74: T32A4 TimmerB compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A4B_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A4BCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   75: T32A4 TimmerB input capture 0
  */
/*--------------------------------------------------*/
void INTT32A4BCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A4BCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   76: T32A4 TimmerB input capture 1
  */
/*--------------------------------------------------*/
void INTT32A4BCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A5AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   77: T32A5 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A5AC_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A5ACCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   78: T32A5 TimmerA/C input capture 0
  */
/*--------------------------------------------------*/
void INTT32A5ACCAP0_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A5ACCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   79: T32A5 TimmerA/C input capture 1
  */
/*--------------------------------------------------*/
void INTT32A5ACCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A5B IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   80: T32A5 TimmerB compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A5B_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A5BCAP0 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   81: T32A5 TimmerB input capture 0
  */
/*--------------------------------------------------*/
void INTT32A5BCAP0_IRQHandle(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTT32A5BCAP1 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   82: T32A5 TimmerB input capture 1
  */
/*--------------------------------------------------*/
void INTT32A5BCAP1_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTPARI IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   83: RAM parity interrupt
  */
/*--------------------------------------------------*/
void INTPARI_IRQHandler(void)
{
    irq_parity();
}

__weak void irq_parity(void){
	
}

/*--------------------------------------------------*/
/** 
  * @brief INTDMAATC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   84: DMA end of transfer
  */
/*--------------------------------------------------*/
void INTDMAATC_IRQHandler(void)
{
    uint32_t i;
    uint32_t flag;
    uint32_t mask;

    /* Read the interrupt flag(IMNFLG3).  */
    flag = TSB_IMN->FLG3;
    for (i=0; i<DMA_CH_NUM; i++)
    {
        /* Check the DMA channel.  */
        mask = (uint32_t)(1 << i);
        if ((flag & mask) == mask)
        {
            /* Clear the interrupt flag. */
            ib_imc_clear_flag(i);
            /* Call Handler. */
            irq_dmac_done(i);
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief INTDMAAERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   85: DMA transfer error
  */
/*--------------------------------------------------*/
void INTDMAAERR_IRQHandler(void)
{
    /* no process */
}

/*--------------------------------------------------*/
/** 
  * @brief INTFLCRDY IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   87: Code FLASH Ready interrupt
  */
/*--------------------------------------------------*/
void INTFLCRDY_IRQHandler(void)
{
    /* no process */
}

/**
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Exported_functions */

/**
 *  @}
 */ /* End of group TMPM4K4A_EVAL */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__BSP_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */
