/** 
 *******************************************************************************
 * @file    bsp.h
 * @brief   Board Support Package External Reference Declaration. \n
 *          TMPM4K4 Evaluation Board.
 * @version V1.0.0.1
 * $Date:: 2018-08-03 19:03:20 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __BSP_H
#define __BSP_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"
//#include "txz_gpio.h"

/**
 *  @addtogroup BSP
 *  @{
 */

/** 
 *  @addtogroup TMPM4K4A_EVAL
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Exported_macro TMPM4K4A Evaluation Board Exported Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Exported_define TMPM4K4A Evaluation Board Exported Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TMPM4K4A_EVAL_Exported_define
 *  @{
 */

/* no define */


/** 
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TMPM4K4A_EVAL_Exported_define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  PORT LED.
*/
/*----------------------------------*/
typedef enum
{
    BSP_LED_0 = 0,          /*!< PORT LED : D10 */
    BSP_LED_1,              /*!< PORT LED : D9  */
    BSP_LED_2,              /*!< PORT LED : D8 */
    BSP_LED_3,              /*!< PORT LED : D7 */
    /*>>> be set bottom */
    BSP_LED_MAX             /*!< max */
    /*<<< be set bottom */
} BSPLed;

/*----------------------------------*/
/** 
 * @brief  PORT Push SW.
*/
/*----------------------------------*/
typedef enum
{
#ifdef T32A_SAMPLE
    BSP_PSW_0 = 0,          /*!< PORT Push SW :S4 */
    BSP_PSW_1,              /*!< PORT Push SW :S5 */
    BSP_PSW_2,              /*!< PORT Push SW :S6 */
    BSP_PSW_3,              /*!< PORT Push SW :S7 */
#else
    BSP_PSW_3 = 0,          /*!< PORT Push SW :S4 */
    BSP_PSW_2,              /*!< PORT Push SW :S5 */
    BSP_PSW_1,              /*!< PORT Push SW :S6 */
    BSP_PSW_0,              /*!< PORT Push SW :S7 */
#endif
    /*>>> be set bottom */
    BSP_PSW_MAX             /*!< max */
    /*<<< be set bottom */
} BSPPsw;

/*----------------------------------*/
/** 
 * @brief  USB UART.
*/
/*----------------------------------*/
typedef enum
{
    BSP_USB_UART_0 = 0,     /*!< USB UART 0 */
    BSP_USB_UART_1 = 1,     /*!< USB UART 1 */
    /*>>> be set bottom */
    BSP_USB_UART_MAX        /*!< max */
    /*<<< be set bottom */
} BSPUsbUart;

/*----------------------------------*/
/** 
 * @brief  Thermistors.
*/
/*----------------------------------*/
typedef enum
{
    BSP_THERMISTORS_0 = 0,  /*!< Thermistors 0 */
    /*>>> be set bottom */
    BSP_THERMISTORS_MAX     /*!< max */
    /*<<< be set bottom */
} BSPThermistors;

/*----------------------------------*/
/** 
 * @brief  Potentiometers.
*/
/*----------------------------------*/
typedef enum
{
    BSP_POTENTIOMETERS_0 = 0,  /*!< Potentiometers 0 */
    BSP_POTENTIOMETERS_1 = 1,  /*!< Potentiometers 1 */
    /*>>> be set bottom */
    BSP_POTENTIOMETERS_MAX     /*!< max */
    /*<<< be set bottom */
} BSPPotentiometers;

/*----------------------------------*/
/** 
 * @brief  External Interrupt.
*/
/*----------------------------------*/
typedef enum
{
    BSP_EXINT_0 = 0,  /*!< low level */
    BSP_EXINT_1 = 1,  /*!<  high level */
    /*>>> be set bottom */
    BSP_EXINT_MAX     /*!< max */
    /*<<< be set bottom */
} BSPExInt;

/*----------------------------------*/
/** 
 * @brief  SPI Flash.
*/
/*----------------------------------*/
typedef enum
{
    BSP_SFLASH_0 = 0,       /*!< SFLASH 0 */
    BSP_SFLASH_1 = 1,       /*!< SFLASH 1 */
    BSP_SFLASH_2 = 2,       /*!< SFLASH 2 */
    BSP_SFLASH_3 = 3,       /*!< SFLASH 3 */
    /*>>> be set bottom */
    BSP_SFLASH_MAX = 4          /*!< max */
    /*<<< be set bottom */
} BSPSFlash;

/*----------------------------------*/
/** 
 * @brief  Timer.
*/
/*----------------------------------*/
typedef enum
{
    BSP_TIMER_1MS = 0,      /*!< 1ms timer */
    BSP_TIMER_1S,           /*!< 1s timer  */
    /*>>> be set bottom */
    BSP_TIMER_MAX           /*!< max */
    /*<<< be set bottom */
} BSPTimer;

/*----------------------------------*/
/** 
 * @brief  Output Pulse.
*/
/*----------------------------------*/
typedef enum
{
    BSP_OUT_PULSE_SAMPLE_A = 0, /*!< pulse A */
    BSP_OUT_PULSE_SAMPLE_B,     /*!< pulse B */
    /*>>> be set bottom */
    BSP_OUT_PULSE_SAMPLE_MAX    /*!< max */
    /*<<< be set bottom */
} BSPOutputPulse;

/*----------------------------------*/
/** 
 * @brief  Timer Counter.
*/
/*----------------------------------*/
typedef enum
{
    BSP_COUNTER_SAMPLE_A = 0, /*!< pulse A */
    BSP_COUNTER_SAMPLE_B,     /*!< pulse B */
    /*>>> be set bottom */
    BSP_COUNTER_SAMPLE_MAX    /*!< max */
    /*<<< be set bottom */
} BSPCounter;
/*----------------------------------*/
/** 
 * @brief  DNF Table Info.
*/
/*----------------------------------*/
typedef enum
{
    BSP_DNF_0 = 0,  /*!< DNF 0 */
    /*>>> be set bottom */
    BSP_DNF_MAX     /*!< max */
    /*<<< be set bottom */
} BSPDnf;
/** 
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Exported_typedef TMPM4K4A Evaluation Board Exported Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Thermistors degC Table Info.
*/
/*----------------------------------*/
typedef struct
{
    int32_t  degC;      /*!< deg.C            */
    uint32_t low;       /*!< AD value :Low    */
    uint32_t center;    /*!< AD value :Center */
    uint32_t high;      /*!< AD value :High   */
} bsp_thermistors_degc_t;

/**
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Exported_typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4K4A_EVAL_Exported_functions TMPM4K4A Evaluation Board Exported Functions
 *  @{
 */

void bsp_initialize(void);
void bsp_finalize(void);
gpio_t *bsp_get_gpio_instance_address(void);
/* led */
uint32_t bsp_get_gpio_group_led(BSPLed led);
uint32_t bsp_get_gpio_num_led(BSPLed led);
/* psw */
uint32_t bsp_get_gpio_group_psw(BSPPsw psw);
uint32_t bsp_get_gpio_num_psw(BSPPsw psw);
/* usb uart */
uint32_t bsp_get_usb_uart_tx_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_tx_dma_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_rx_dma_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_tx_baudrate(BSPUsbUart uart);
IRQn_Type bsp_get_usb_uart_tx_nvic(BSPUsbUart uart);
IRQn_Type bsp_get_usb_uart_rx_nvic(BSPUsbUart uart);
IRQn_Type bsp_get_usb_uart_err_nvic(BSPUsbUart uart);
/* tspi */
uint32_t bsp_get_tspi_tx_ch(BSPSFlash tspi);
IRQn_Type bsp_get_tspi_tx_nvic(BSPSFlash tspi);
/* timer */
void bsp_get_timer(BSPTimer timer, uint32_t *p_ch, uint32_t *p_type);
uint32_t bsp_get_timer_ch(BSPTimer timer);
uint32_t bsp_get_timer_type(BSPTimer timer);
IRQn_Type bsp_get_timer_nvic(BSPTimer timer);
/* output pulse */
uint32_t bsp_get_output_pulse_ch(BSPOutputPulse pulse);
uint32_t bsp_get_output_pulse_type(BSPOutputPulse pulse);
IRQn_Type bsp_get_output_pulse_nvic(BSPOutputPulse pulse);
/* thermistors */
/* potentiometers */
uint32_t bsp_get_potentiometers_adin(BSPPotentiometers potentiometers);
uint32_t bsp_get_potentiometers_ch(BSPPotentiometers potentiometers);
/* external interrupt */
void bsp_irq_exint_enable(void);
void bsp_irq_exint_disable(void);
IRQn_Type bsp_get_exint_nvic(void);
void bsp_irq_dnf_enable(void);
void bsp_irq_dnf_disable(void);
TSB_DNF_TypeDef * bsp_get_dnf_reg(BSPDnf dnf);
uint32_t bsp_get_dnf_int_num(BSPDnf dnf);
IRQn_Type bsp_get_dnf_nvic(BSPDnf dnf);
/* rmc */
void bsp_irq_rmc_enable(void);
void bsp_irq_rmc_disable(void);

/*>>> Add to processing by user. */
extern void irq_systick(void);
extern void irq_psw(BSPPsw psw);
extern void irq_potentiometers_sgl(BSPPotentiometers potentiometers);
extern void irq_potentiometers_cnt(BSPPotentiometers potentiometers);
extern void irq_sflash_rx(BSPSFlash sflash);
extern void irq_sflash_tx(BSPSFlash sflash);
extern void irq_sflash_err(BSPSFlash sflash);
extern void irq_usb_uart_rx(BSPUsbUart uart);
extern void irq_usb_uart_tx(BSPUsbUart uart);
extern void irq_usb_uart_err(BSPUsbUart uart);
extern void irq_timer(BSPTimer timer);
extern void irq_counter(BSPCounter cnt);
extern void irq_dmac_done(uint32_t ch);
extern void irq_dmac_err(void);
extern void irq_rmc(void);
extern void irq_exint(BSPExInt exi_p);
extern void irq_i2c(void);
extern void irq_adc_sgl(void);
extern void irq_adc_cnt(void);
extern void irq_adc_trig(void);
extern void irq_parity(void);
/*<<< Add to processing by user. */

/** 
 *  @}
 */ /* End of group TMPM4K4A_EVAL_Exported_functions */

/**
 *  @}
 */ /* End of group TMPM4K4A_EVAL */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BSP_H */

