/**
 *******************************************************************************
 * @file     fc.h
 * @brief    This file provides API functions for FC driver.
 * @version  V1.0.3.0
 * $date::                 $
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __FC_H
#define __FC_H

#ifdef __cplusplus
extern "C" {
#endif                          /* __cplusplus */

/* Includes ------------------------------------------------------------------*/
#include "txz_driver_def.h"

#if defined(__FC_H)
/** @addtogroup Periph driver
  * @{
  */

/** @addtogroup fc
  * @brief fc
  * @{
  */


/** @addtogroup FlashApi_Exported_Types
  * @{
  */

/** 
 *  @enum	fc_sr0_t
 *  @brief	Enumerated type definition of the FCSR0 register.
 */
typedef enum {
	FC_SR0_RDYBSY		= (0x00000001UL)	/*!< 0:Busy, 1:Ready all flash		*/
} fc_sr0_t;

/*----------------------------------*/
/** 
 *  @enum	fc_erase_kind_t
 *  @brief  FC Erase Flash Kind structure definenition.
*/
/*----------------------------------*/
typedef enum {
	FC_ERASE_KIND_PAGE		= (0x00000040UL)	/*!< Page Erase						*/
} fc_erase_kind_t;

typedef enum {
	FC_CODE_FLASH_PAGE0	= 0,	/*!< (0x5E000000UL), CODE FLASH Page0 */
	FC_CODE_FLASH_PAGE1,		/*!< (0x5E001000UL), CODE FLASH Page1 */
	FC_CODE_FLASH_PAGE2,		/*!< (0x5E002000UL), CODE FLASH Page2 */
	FC_CODE_FLASH_PAGE3,		/*!< (0x5E003000UL), CODE FLASH Page3 */
	FC_CODE_FLASH_PAGE4,		/*!< (0x5E004000UL), CODE FLASH Page4 */
	FC_CODE_FLASH_PAGE5,		/*!< (0x5E005000UL), CODE FLASH Page5 */
	FC_CODE_FLASH_PAGE6,		/*!< (0x5E006000UL), CODE FLASH Page6 */
	FC_CODE_FLASH_PAGE7,		/*!< (0x5E007000UL), CODE FLASH Page7 */
	FC_CODE_FLASH_PAGE8,		/*!< (0x5E008000UL), CODE FLASH Page8 */
	FC_CODE_FLASH_PAGE9,		/*!< (0x5E009000UL), CODE FLASH Page9 */
	FC_CODE_FLASH_PAGE10,		/*!< (0x5E00A000UL), CODE FLASH Page10 */
	FC_CODE_FLASH_PAGE11,		/*!< (0x5E00B000UL), CODE FLASH Page11 */
	FC_CODE_FLASH_PAGE12,		/*!< (0x5E00C000UL), CODE FLASH Page12 */
	FC_CODE_FLASH_PAGE13,		/*!< (0x5E00D000UL), CODE FLASH Page13 */
	FC_CODE_FLASH_PAGE14,		/*!< (0x5E00E000UL), CODE FLASH Page14 */
	FC_CODE_FLASH_PAGE15,		/*!< (0x5E00F000UL), CODE FLASH Page15 */
	FC_CODE_FLASH_PAGE16,		/*!< (0x5E010000UL), CODE FLASH Page16 */
	FC_CODE_FLASH_PAGE17,		/*!< (0x5E011000UL), CODE FLASH Page17 */
	FC_CODE_FLASH_PAGE18,		/*!< (0x5E012000UL), CODE FLASH Page18 */
	FC_CODE_FLASH_PAGE19,		/*!< (0x5E013000UL), CODE FLASH Page19 */
	FC_CODE_FLASH_PAGE20,		/*!< (0x5E014000UL), CODE FLASH Page20 */
	FC_CODE_FLASH_PAGE21,		/*!< (0x5E015000UL), CODE FLASH Page21 */
	FC_CODE_FLASH_PAGE22,		/*!< (0x5E016000UL), CODE FLASH Page22 */
	FC_CODE_FLASH_PAGE23,		/*!< (0x5E017000UL), CODE FLASH Page23 */
	FC_CODE_FLASH_PAGE24,		/*!< (0x5E018000UL), CODE FLASH Page24 */
	FC_CODE_FLASH_PAGE25,		/*!< (0x5E019000UL), CODE FLASH Page25 */
	FC_CODE_FLASH_PAGE26,		/*!< (0x5E01A000UL), CODE FLASH Page26 */
	FC_CODE_FLASH_PAGE27,		/*!< (0x5E01B000UL), CODE FLASH Page27 */
	FC_CODE_FLASH_PAGE28,		/*!< (0x5E01C000UL), CODE FLASH Page28 */
	FC_CODE_FLASH_PAGE29,		/*!< (0x5E01D000UL), CODE FLASH Page29 */
	FC_CODE_FLASH_PAGE30,		/*!< (0x5E01E000UL), CODE FLASH Page30 */
	FC_CODE_FLASH_PAGE31,		/*!< (0x5E01F000UL), CODE FLASH Page31 */
#if defined(TMPM4K4A)
	FC_CODE_FLASH_PAGE32,		/*!< (0x5E020000UL), CODE FLASH Page32 */
	FC_CODE_FLASH_PAGE33,		/*!< (0x5E021000UL), CODE FLASH Page33 */
	FC_CODE_FLASH_PAGE34,		/*!< (0x5E022000UL), CODE FLASH Page34 */
	FC_CODE_FLASH_PAGE35,		/*!< (0x5E023000UL), CODE FLASH Page35 */
	FC_CODE_FLASH_PAGE36,		/*!< (0x5E024000UL), CODE FLASH Page36 */
	FC_CODE_FLASH_PAGE37,		/*!< (0x5E025000UL), CODE FLASH Page37 */
	FC_CODE_FLASH_PAGE38,		/*!< (0x5E026000UL), CODE FLASH Page38 */
	FC_CODE_FLASH_PAGE39,		/*!< (0x5E027000UL), CODE FLASH Page39 */
	FC_CODE_FLASH_PAGE40,		/*!< (0x5E028000UL), CODE FLASH Page40 */
	FC_CODE_FLASH_PAGE41,		/*!< (0x5E029000UL), CODE FLASH Page41 */
	FC_CODE_FLASH_PAGE42,		/*!< (0x5E02A000UL), CODE FLASH Page42 */
	FC_CODE_FLASH_PAGE43,		/*!< (0x5E02B000UL), CODE FLASH Page43 */
	FC_CODE_FLASH_PAGE44,		/*!< (0x5E02C000UL), CODE FLASH Page44 */
	FC_CODE_FLASH_PAGE45,		/*!< (0x5E02D000UL), CODE FLASH Page45 */
	FC_CODE_FLASH_PAGE46,		/*!< (0x5E02E000UL), CODE FLASH Page46 */
	FC_CODE_FLASH_PAGE47,		/*!< (0x5E02F000UL), CODE FLASH Page47 */
	FC_CODE_FLASH_PAGE48,		/*!< (0x5E030000UL), CODE FLASH Page48 */
	FC_CODE_FLASH_PAGE49,		/*!< (0x5E031000UL), CODE FLASH Page49 */
	FC_CODE_FLASH_PAGE50,		/*!< (0x5E032000UL), CODE FLASH Page50 */
	FC_CODE_FLASH_PAGE51,		/*!< (0x5E033000UL), CODE FLASH Page51 */
	FC_CODE_FLASH_PAGE52,		/*!< (0x5E034000UL), CODE FLASH Page52 */
	FC_CODE_FLASH_PAGE53,		/*!< (0x5E035000UL), CODE FLASH Page53 */
	FC_CODE_FLASH_PAGE54,		/*!< (0x5E036000UL), CODE FLASH Page54 */
	FC_CODE_FLASH_PAGE55,		/*!< (0x5E037000UL), CODE FLASH Page55 */
	FC_CODE_FLASH_PAGE56,		/*!< (0x5E038000UL), CODE FLASH Page56 */
	FC_CODE_FLASH_PAGE57,		/*!< (0x5E039000UL), CODE FLASH Page57 */
	FC_CODE_FLASH_PAGE58,		/*!< (0x5E03A000UL), CODE FLASH Page58 */
	FC_CODE_FLASH_PAGE59,		/*!< (0x5E03B000UL), CODE FLASH Page59 */
	FC_CODE_FLASH_PAGE60,		/*!< (0x5E03C000UL), CODE FLASH Page60 */
	FC_CODE_FLASH_PAGE61,		/*!< (0x5E03D000UL), CODE FLASH Page61 */
	FC_CODE_FLASH_PAGE62,		/*!< (0x5E03E000UL), CODE FLASH Page62 */
	FC_CODE_FLASH_PAGE63		/*!< (0x5E03F000UL), CODE FLASH Page63 */
#endif
} fc_code_flash_page_number_t;

/** @} */
/* End of group FlashApi_Exported_Types */

/** @addtogroup FlashApi_Exported_Constants
  * @{
  */


#define FC_CODE_FLASH_ADDRESS_TOP	(0x5E000000UL)   		/*!< CODE FLASH Address Top */
#if defined(TMPM4K4A)
    #define FC_CODE_FLASH_ADDRESS_END  	(0x5E03FFFFUL)   		/*!< CODE FLASH Address End */
#else
    #define FC_CODE_FLASH_ADDRESS_END  	(0x5E01FFFFUL)   		/*!< CODE FLASH Address End */
#endif
#define FC_PAGE_SIZE				(0x1000)				/*!< The number of bytes in a page. */
#define FC_CODE_FLASH_WRITE_SIZE	(sizeof(uint32_t)*4)	/*!< CODE FLASH WRITE SIZE */

/** @} */
/* End of group FlashApi_Exported_Constants */



/** @weakgroup FlashApi_Exported_FunctionPrototypes
  * @{
  */
TXZ_WorkState fc_get_status(fc_sr0_t status);
TXZ_Result fc_write_code_flash(uint8_t* src_address, uint8_t* dst_address, uint32_t size);
TXZ_Result fc_erase_page_code_flash(fc_code_flash_page_number_t first_page, uint8_t num_of_pages);
TXZ_Result fc_blank_check_page_code_flash(fc_code_flash_page_number_t first_page, fc_code_flash_page_number_t lasr_page);

/** @} */
/* End of group FlashApi_Exported_FunctionPrototypes */


/** @} */
/* End of group fc */

/** @} */
/* End of group Periph_driver */

#endif                          /* defined(__FC_H) */

#ifdef __cplusplus
}
#endif                          /* __cplusplus */
#endif                          /* __FC_H */
