/** 
 *******************************************************************************
 * @file    main.c
 * @brief   Main function of the Code Flash E2PROM Emulation Sample Application.
 * @version V1.0.1.0
 * $Date:: 2017-11-01 15:34:32 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup CODE_FLASH_E2PROM_EMULATION
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "txz_sample_def.h"
#if defined(TMPM4K4A)
#include "fc.h"
#endif

/*------------------------------------------------------------------------------*/
/*  Definition                                                  */
/*------------------------------------------------------------------------------*/
/* User Setting */
#define CF_E2ROM_EMU_DATA_SIZE					(100)				/*!< Writing data size. Max:CF_E2ROM_EMU_DATA_SIZE_MAX(4079 bytes) */

/* Constant */
#define CF_E2ROM_EMU_DATA_SIZE_MAX				(4079)				/*!< bytes. Maximum data size of one page.    */

#define CF_E2ROM_EMU_RECORD_D_REWRITE_MAX		((uint32_t)0x00002710)		/*!< The maximum number of rewrite of the page. */

#define CF_E2ROM_EMU_RECORD_D_SIZE_MAX			FC_CODE_FLASH_WRITE_SIZE	/*!< The Record-D maximum size. */

#define CF_E2ROM_EMU_WRITTEN_MARK				((uint8_t)0xa5)				/*!< Wrtten mark    */

#define CF_E2ROM_EMU_DATA_REMAINDER				(CF_E2ROM_EMU_DATA_SIZE - (CF_E2ROM_EMU_DATA_SIZE / FC_CODE_FLASH_WRITE_SIZE) * FC_CODE_FLASH_WRITE_SIZE)
#if (CF_E2ROM_EMU_DATA_SIZE - (CF_E2ROM_EMU_DATA_SIZE / 16)  * 16)
#define CF_E2ROM_EMU_RECORD_SIZE				(uint16_t)(CF_E2ROM_EMU_DATA_SIZE + FC_CODE_FLASH_WRITE_SIZE - CF_E2ROM_EMU_DATA_REMAINDER)				/*!< When a remainder does not equal 0, the manegement area is a remainder area    */
#else
#define CF_E2ROM_EMU_RECORD_SIZE				(uint16_t)(CF_E2ROM_EMU_DATA_SIZE + FC_CODE_FLASH_WRITE_SIZE)		/*!< When a remainder equals 0, it adds the manegemt area    */
#endif

#define CF_E2ROM_EMU_NUM_OF_RECORD				(CF_E2ROM_EMU_DATA_SIZE_MAX / CF_E2ROM_EMU_RECORD_SIZE)

#define CF_E2ROM_EMU_RECORD_D_WRITE_POSITION	(FC_PAGE_SIZE - 16)
#define CF_E2ROM_EMU_RECORD_D_READ_POSITION		(FC_PAGE_SIZE - 4)

/* Varibale */

/* Registers */
/* OSCF     */
#define CGOSCCR_OSCF_MASK               ((uint32_t)0x00000200)    /*!< OSCF      :Mask                  */
#define CGOSCCR_OSCF_R_IHOSC            ((uint32_t)0x00000000)    /*!< OSCF      :[R]   :Internal HOSC  */
#define CGOSCCR_OSCF_R_EHOSC            ((uint32_t)0x00000200)    /*!< OSCF      :[R]   :External HOSC  */
/* IHOSC1EN */
#define CGOSCCR_IHOSC1EN_MASK           ((uint32_t)0x00000001)    /*!< IHOSC1EN  :Mask                  */
#define CGOSCCR_IHOSC1EN_RW_DISABLE     ((uint32_t)0x00000000)    /*!< IHOSC1EN  :[R/W] :Disable        */
#define CGOSCCR_IHOSC1EN_RW_ENABLE      ((uint32_t)0x00000001)    /*!< IHOSC1EN  :[R/W] :Enable         */
/*  */
#define CGOSCCR_IHOSC1F_MASK			((uint32_t)0x00010000)    /*!< IHOSC1EN  :Mask                  */
#define CGOSCCR_IHOSC1F_R_BUSY			((uint32_t)0x00000000)    /*!< IHOSC1EN  :[R]Busy               */
#define CGOSCCR_IHOSC1F_R_DONE			((uint32_t)0x00010000)    /*!< IHOSC1EN  :[R]Done               */
/* CORE  */
#define CORE_PRIMASK_INT_DISABLE		((uint32_t)0x00000000)    /*!< Disables interruption. PRIMASK=0	*/
#define CORE_PRIMASK_INT_ENABLE			((uint32_t)0x00000001)    /*!< Enables interruption. PRIMASK=1	*/

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  External Definition                                                        */
/*------------------------------------------------------------------------------*/
#if     defined ( __CC_ARM )    /* ARM Compiler */
extern unsigned int Load$$FLASH_CONTROL$$Base;
extern unsigned int Image$$FLASH_CONTROL$$Base;
extern unsigned int Image$$FLASH_CONTROL$$Limit;
extern unsigned int Image$$ER_IROM1$$Base;
#elif   defined ( __ICCARM__ )  /* IAR Compiler */
#pragma section = "FLASH_CONTROL_ROM"
#pragma section = "FLASH_CONTROL_RAM"
#pragma section = "__ICFEDIT_intvec_start__"
#endif

/*----------------------------------*/
/** 
 * @brief CFE2ROMEMU User & manegement data.
*/
/*----------------------------------*/
typedef union {
	uint8_t user_data[CF_E2ROM_EMU_DATA_SIZE];		/*!< User data area for the E2PROM Emulaton.	*/ 
	uint8_t record_data[CF_E2ROM_EMU_RECORD_SIZE];	/*!< User must not use it. User data and manegement area for the E2PROM Emulaton. */
} cf_e2prom_emu_user_data_t;

/*----------------------------------*/
/** 
 * @brief CFE2ROMEMU Initialization Information for API.
*/
/*----------------------------------*/
typedef struct
{
	fc_code_flash_page_number_t first_page;	/*!< First page number of the code flash */
	uint8_t number_of_pages;				/*!< The number of pages of the code flash to use */
	uint16_t data_size;						/*!< Writing data size. Max:4079 bytes */
	uint8_t* data_address;					/*!< Data address in the RAM area */
} cf_e2prom_emu_init_t;

/*----------------------------------*/
/** 
 * @brief CFE2ROMEMU Instance Information.
*/
/*----------------------------------*/
typedef struct
{
	fc_code_flash_page_number_t first_page;		/*!< First page number of the code flash */
	fc_code_flash_page_number_t last_page;		/*!< Last page number of the code flash */
	fc_code_flash_page_number_t current_page;	/*!< Current page */
	uint8_t* data_address;						/*!< Data address in the RAM area */
	uint16_t data_size;							/*!< Writing data size. Max:4080 bytes */
	uint16_t current_record;					/*!< Current record */
	uint16_t recod_size;						/*!< Record size */
} cf_e2prom_emu_instance_t;

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/

static cf_e2prom_emu_user_data_t user_data;	/*!< User Data. Not use the record_data.	*/
static cf_e2prom_emu_instance_t cf_e2prom_emu_instance;		/*!< Instance of E2ROM Emulation.	*/


/*------------------------------------------------------------------------------*/
/*  Constant Declaration                                                               */
/*------------------------------------------------------------------------------*/
static const uint8_t test_user_data[CF_E2ROM_EMU_DATA_SIZE] = {
		0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,	0x08,0x09,0x10,0x11,0x12,0x13,0x14,0x15,
		0x16,0x17,0x18,0x19,0x20,0x21,0x22,0x23,	0x24,0x25,0x26,0x27,0x28,0x29,0x30,0x31,
		0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,	0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,
		0x48,0x49,0x50,0x51,0x52,0x53,0x54,0x55,	0x56,0x57,0x58,0x59,0x60,0x61,0x62,0x63,
		0x64,0x65,0x66,0x67,0x68,0x69,0x70,0x71,	0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,
		0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,	0x88,0x89,0x90,0x91,0x92,0x93,0x94,0x95,
		0x96,0x97,0x98,0x99
};

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static void cf_e2prom_emu_internal_oscillator_enable(void);
static void cf_e2prom_emu_ramload(void);
static TXZ_Result cf_e2prom_emu_initialize(cf_e2prom_emu_init_t* init);
static TXZ_Result cf_e2prom_emu_blank_check_using_pages(void);
static TXZ_Result cf_e2prom_emu_format(void);
static TXZ_Result cf_e2prom_emu_restore(void);
static void cf_e2prom_emu_read_data(void);
static uint32_t cf_e2prom_emu_read_record_d(fc_code_flash_page_number_t page);
static uint8_t cf_e2prom_emu_read_written_mark(fc_code_flash_page_number_t page, uint16_t record);
static uint8_t cf_e2prom_emu_read_written_mark_current_record(void);
static TXZ_Result cf_e2prom_emu_write_record(void);

/*--------------------------------------------------*/
/** 
  * @brief  Enables the internal high speed osillator.
  * @param  -
  * @return -.
  * @note   Provides to the internal high speed clock of a 10MHz.
  */
/*--------------------------------------------------*/
static void cf_e2prom_emu_internal_oscillator_enable(void)
{
    uint32_t work;

    /* [CGOSCCR]<IHOSC1EN> : Enables the IHOSC1    */
    work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC1EN_MASK);
    TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC1EN_RW_ENABLE);
	/*  [CGOSCCR]<HOSC1F> : HOSC1F confirm */
	while(1){
		work = (uint32_t)(TSB_CG->OSCCR  & CGOSCCR_IHOSC1F_MASK);
		if(work == (uint32_t)(CGOSCCR_IHOSC1F_R_DONE)){
			break;
		}
	}
}

/*--------------------------------------------------*/
/** 
  * @brief  Load codes of the flash contorl program into RAM.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void cf_e2prom_emu_ramload(void)
{
	unsigned int *src;
	unsigned int *dst;
	unsigned int len;

#if     defined ( __CC_ARM )    /* ARM Compiler */
	src = (unsigned int *)&Load$$FLASH_CONTROL$$Base;		/* A program transfer source address (An address of the load image in the scatter file.) */
	dst = (unsigned int *)&Image$$FLASH_CONTROL$$Base;		/* A program transfer destination address(An address of execution image in the scatter file.) */
	len = (unsigned int)((unsigned long)&Image$$FLASH_CONTROL$$Limit - (unsigned long)dst);
#elif   defined ( __ICCARM__ )  /* IAR Compiler */
	src = (unsigned int *)((uint32_t *)__section_begin("FLASH_CONTROL_ROM"));		/* A program transfer source address (An address of the load image in the scatter file.) */
	dst = (unsigned int *)((uint32_t *)__section_begin("FLASH_CONTROL_RAM"));		/* A program transfer destination address(An address of execution image in the scatter file.) */
	len = (unsigned int)((uint32_t)__section_size("FLASH_CONTROL_RAM"));
#endif
	while (len > 0) {
		*dst = *src;
		dst++;
		src++;
		len -= sizeof(int);
	}

	/* If it works a program on the RAM immediately, these commands are necessary. */
	/* It carries out with the sample program just in case. */
#if 1
        __DSB();		/* This command outputs a data from a buffer forcibly. */
        __ISB();		/* It fhushes the pipeline of the CPU. */
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  Code Flash E2PROM Emulation Initialize.
  * @param  init
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   Fills it up with 0xFF for record size form the address of data_address. 
  */
/*--------------------------------------------------*/
static TXZ_Result cf_e2prom_emu_initialize(cf_e2prom_emu_init_t* init)
{
	TXZ_Result result = TXZ_SUCCESS;

	/* Instance Clear */
	(void)memset(&cf_e2prom_emu_instance, 0x00, sizeof(cf_e2prom_emu_instance));
	
	if(init->data_size <= CF_E2ROM_EMU_DATA_SIZE_MAX){	/* Check a data size */
		cf_e2prom_emu_instance.first_page = init->first_page;
		cf_e2prom_emu_instance.last_page = (fc_code_flash_page_number_t)(init->first_page + init->number_of_pages -1);
		cf_e2prom_emu_instance.data_size = init->data_size;
		cf_e2prom_emu_instance.data_address = init->data_address;

		/* Recode Structer of the Code Flash ROM														*/
		/* User Data																					*/
		/* |-------- n byte(init.data_size) -------|													*/
		/* 																								*/
		/* Add write mark(1byte)																		*/
		/* when a multiple of 16bytes, the write mark is last bytes of adding 16bytes					*/
		/* |-------- n byte(init.data_size) -------| ----- 16bytes -----|								*/
		/*																								*/
		/* when a multiple of 16bytes, the write mark is last bytes of adding fields from 1 to 15bytes	*/
		/* |-------- n byte(init.data_size) -------| --- 1-15bytes ---|    								*/

		cf_e2prom_emu_instance.recod_size = CF_E2ROM_EMU_RECORD_SIZE;
		(void)memset(cf_e2prom_emu_instance.data_address, 0xFFFFFFFUL, CF_E2ROM_EMU_RECORD_SIZE);
		{
			cf_e2prom_emu_instance.data_address[CF_E2ROM_EMU_RECORD_SIZE - 1] = CF_E2ROM_EMU_WRITTEN_MARK;
		}
		
		/* Copis the fc code to internal RAM */
		cf_e2prom_emu_ramload();
	}
	else{
		result = TXZ_ERROR;
	}

	return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check all pages of the code Flash E2PROM emulation area
  *			whether it is a blank.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It erases all of code flash E2PROM emulation
  *         areas.
  */
/*--------------------------------------------------*/
static TXZ_Result cf_e2prom_emu_blank_check_using_pages(void)
{
	/* Blank Check */
	TXZ_Result result = fc_blank_check_page_code_flash(cf_e2prom_emu_instance.first_page, cf_e2prom_emu_instance.last_page);

	return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Formats the code Flash E2PROM emulation area.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It erases all of code flash E2PROM emulation
  *         areas.
  */
/*--------------------------------------------------*/
static TXZ_Result cf_e2prom_emu_format(void)
{
	TXZ_Result result = TXZ_SUCCESS;

	/* Disables the interruption. PRIMASK=0 */
	__set_PRIMASK(CORE_PRIMASK_INT_DISABLE);
	
	/* Moves the vector table to internal RAM. */
	/* Lower 6bits of VTOR is a reservation. */
#if     defined ( __CC_ARM )    /* ARM Compiler */
	SCB->VTOR = (uint32_t)&Image$$FLASH_CONTROL$$Base;
#elif   defined ( __ICCARM__ )  /* IAR Compiler */
        SCB->VTOR = (uint32_t)((uint32_t *)__section_begin("FLASH_CONTROL_RAM"));
#endif
	{
		/* Writes erasure number of times 0 in the D-Record of the first page. */
		uint8_t src[FC_CODE_FLASH_WRITE_SIZE] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,  0xFF, 0xFF, 0xFF, 0xFF, 0x10, 0x27, 0x00, 0x00};
		uint8_t* dst_address = (uint8_t*)(FC_CODE_FLASH_ADDRESS_TOP + ((uint32_t)cf_e2prom_emu_instance.first_page * (uint32_t)FC_PAGE_SIZE) + CF_E2ROM_EMU_RECORD_D_WRITE_POSITION);
		result = fc_write_code_flash(src, dst_address, (uint32_t)FC_CODE_FLASH_WRITE_SIZE);

		if(result == TXZ_SUCCESS){
			/* Registers current page and record.  */
			cf_e2prom_emu_instance.current_page = cf_e2prom_emu_instance.first_page;
			cf_e2prom_emu_instance.current_record = (uint16_t)0;
		}
	}

	/* Returns the vector table to internal ROM. */
	/* Lower 6bits of VTOR is a reservation. */
#if     defined ( __CC_ARM )    /* ARM Compiler */
	SCB->VTOR = (uint32_t)&Image$$ER_IROM1$$Base;
#elif   defined ( __ICCARM__ )  /* IAR Compiler */
        SCB->VTOR = (uint32_t)((uint32_t *)__section_begin("__ICFEDIT_intvec_start__"));
#endif


	/* Enables the interruption. */
	__set_PRIMASK(CORE_PRIMASK_INT_ENABLE);

	return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Restore the management state of code Flash E2PROM emulation befor resetting it.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result cf_e2prom_emu_restore(void)
{
	TXZ_Result result = TXZ_SUCCESS;

	/* When the the current_page is 0xFFFFFFFF, the current page is not decided.*/
	cf_e2prom_emu_instance.current_page	= (fc_code_flash_page_number_t)(0xFFFFFFFFUL);

	/* When the the current record is 0xFFFF, the current record are not decided.*/
	cf_e2prom_emu_instance.current_record = (uint16_t)0xFFFF;

	/************************************************/
	/* Decides current page							*/
	/************************************************/
	/* When the number of pages is one,  a page is decided.*/
	if(cf_e2prom_emu_instance.first_page == cf_e2prom_emu_instance.last_page){
		cf_e2prom_emu_instance.current_page = cf_e2prom_emu_instance.first_page;
	}
	else{
		fc_code_flash_page_number_t page;
		uint32_t record_d_1;

		/* It looks for which page a last written record is. */
		/* Increments a page number from the first paget */
		record_d_1 = cf_e2prom_emu_read_record_d(cf_e2prom_emu_instance.first_page);
		for(page=cf_e2prom_emu_instance.first_page; page<=cf_e2prom_emu_instance.last_page-1; page++){

			/* When the page yet has been never used, */
			if(record_d_1 == (uint32_t)(0xFFFFFFFFUL)){
				/* Incase of first page. */
				if(page == cf_e2prom_emu_instance.first_page){
					/* When the last page yet has been never used too, */
					uint32_t record_d_last = cf_e2prom_emu_read_record_d(cf_e2prom_emu_instance.last_page);
					if(record_d_last == (uint32_t)(0xFFFFFFFFUL)){
						/* the current page is a firstlast page. */
						cf_e2prom_emu_instance.current_page = cf_e2prom_emu_instance.first_page;
					}
					else{
						/* the current page is a last page. */
						cf_e2prom_emu_instance.current_page = cf_e2prom_emu_instance.last_page;
					}
				}
				/* In case of not first page. */
				else{
					/* the current page is a page before one. */
					cf_e2prom_emu_instance.current_page = (fc_code_flash_page_number_t)(page-1);
				}
				break;
			}

			/* When the page already has been used, */
			else{
				uint32_t record_d_2 = cf_e2prom_emu_read_record_d((fc_code_flash_page_number_t)(page+1));
				/* If the  Record-D of this page is greater than one of next page, it decides the current page. */
				if((record_d_2 == (uint32_t)(0xFFFFFFFFUL)) || (record_d_1 > record_d_2)){
					cf_e2prom_emu_instance.current_page = page;
					break;
				}
				/* Furthermore, if this page is a page before last one, the current page is the last page. */
				if(page == cf_e2prom_emu_instance.last_page-1){
					cf_e2prom_emu_instance.current_page = cf_e2prom_emu_instance.last_page;
					break;
				}
			}
		}
		if(cf_e2prom_emu_instance.current_page == (fc_code_flash_page_number_t)(0xFFFFFFFFUL)){
			/* The current page was not decided. */
			result = TXZ_ERROR;
		}
	}

	/************************************************/
	/* Decides current record in the current page.	*/
	/************************************************/
	if(result == TXZ_SUCCESS){
		uint8_t written_mark = cf_e2prom_emu_read_written_mark(cf_e2prom_emu_instance.current_page, (uint16_t)0);

		/* In the case of the page which has yet written no record, the written mark is a value of 0xFF. */
		/* Then the current record is first record - zeroth-. */
		if(written_mark == (uint8_t)0xFF){
			cf_e2prom_emu_instance.current_record = (uint16_t)0;
		}
		else{
			uint16_t record;
			/* Seaches the current record. */
			for(record = (uint16_t)1; record < CF_E2ROM_EMU_NUM_OF_RECORD; record++){
				written_mark = cf_e2prom_emu_read_written_mark(cf_e2prom_emu_instance.current_page, record);
				if(written_mark != CF_E2ROM_EMU_WRITTEN_MARK){
					/* the current record is a record before one. */
					cf_e2prom_emu_instance.current_record = (uint16_t)(record - 1);
					break;
				}
			}
			/* If all records are written, current record is last record.  */
			if(record == CF_E2ROM_EMU_NUM_OF_RECORD){
				cf_e2prom_emu_instance.current_record = (uint16_t)(CF_E2ROM_EMU_NUM_OF_RECORD - 1);
			}
		}
		if(cf_e2prom_emu_instance.current_record == (uint16_t)0xFFFF){
			/* The current record was not decided. */
			result = TXZ_ERROR;
		}
	}

	return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Reads user data of code Flash E2PROM emulation areas.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void cf_e2prom_emu_read_data(void)
{
	uint8_t* src_address = (uint8_t*)((uint32_t)cf_e2prom_emu_instance.current_page * (uint32_t)FC_PAGE_SIZE) + (cf_e2prom_emu_instance.current_record * CF_E2ROM_EMU_RECORD_SIZE);
	(void)memcpy(cf_e2prom_emu_instance.data_address, src_address, cf_e2prom_emu_instance.data_size);
}

/*--------------------------------------------------*/
/** 
  * @brief  Writes the record to code Flash E2PROM emulation areas.
  * @param  page
  * @return Record-D.
  * @retval Value of Record-D
  * @note   -
  */
/*--------------------------------------------------*/
static uint32_t cf_e2prom_emu_read_record_d(fc_code_flash_page_number_t page)
{
	/* Record-D address in a page */
	uint32_t* addr =  (uint32_t*)((uint32_t)(page * FC_PAGE_SIZE) + CF_E2ROM_EMU_RECORD_D_READ_POSITION);

	/* Read and return the Record-D */
	return(*addr);
}

/*--------------------------------------------------*/
/** 
  * @brief  Reads the written mark each page and each recode.
  * @param  page         : page_num
  * @param  record       : record_num
  * @return Written mark.
  * @retval Value of written mark.
  * @note   -
  */
/*--------------------------------------------------*/
static uint8_t cf_e2prom_emu_read_written_mark(fc_code_flash_page_number_t page, uint16_t record)
{
	/* Written mark address in page,record */
	uint8_t* addr = (uint8_t*)(((uint32_t)page * (uint32_t)FC_PAGE_SIZE)
					+ (record * CF_E2ROM_EMU_RECORD_SIZE)
					+ (CF_E2ROM_EMU_RECORD_SIZE - 1));

	/* Read and return the written mark */
	return(*addr);
}

/*--------------------------------------------------*/
/** 
  * @brief  Reads the written current page and current record.
  * @param  -
  * @return Written mark.
  * @retval Value of written mark.
  * @note   -
  */
/*--------------------------------------------------*/
static uint8_t cf_e2prom_emu_read_written_mark_current_record(void)
{
	/* Reads the written current page and current record. */
	/* Read and return the written mark */
	return(cf_e2prom_emu_read_written_mark(cf_e2prom_emu_instance.current_page, cf_e2prom_emu_instance.current_record));
}

/*--------------------------------------------------*/
/** 
  * @brief  Writes the record to code Flash E2PROM emulation areas.
  * @param  -
  * @return TXZ_Result
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result cf_e2prom_emu_write_record(void)
{
	TXZ_Result result = TXZ_SUCCESS;

	/* If a data of the user data area is same as it in code flash, it will not write the user data. */
	{
		uint8_t* flash_address = (uint8_t*)(FC_CODE_FLASH_ADDRESS_TOP
									+ (uint32_t)(cf_e2prom_emu_instance.current_page * FC_PAGE_SIZE)
									+ (uint32_t)(cf_e2prom_emu_instance.current_record * CF_E2ROM_EMU_RECORD_SIZE));
		int retval = memcmp(flash_address, cf_e2prom_emu_instance.data_address, cf_e2prom_emu_instance.data_size);
		if(retval == 0){
			return(TXZ_SUCCESS);
		}
	}

	/* Disables the interruption. PRIMASK=0 */
	__set_PRIMASK(CORE_PRIMASK_INT_DISABLE);

	/* Moves the vector table to internal RAM. */
	/* Lower 6bits of VTOR is a reservation. */ 
#if     defined ( __CC_ARM )    /* ARM Compiler */
	SCB->VTOR = (uint32_t)&Image$$FLASH_CONTROL$$Base;
#elif   defined ( __ICCARM__ )  /* IAR Compiler */
        SCB->VTOR = (uint32_t)((uint32_t *)__section_begin("FLASH_CONTROL_RAM"));
#endif

	/* Decides the record to write. */
	/* In clase of the flash ROM is written more than once, it prepares to write to the next recored.  */
	if((cf_e2prom_emu_read_written_mark(cf_e2prom_emu_instance.current_page, (uint16_t)0x00) != (uint8_t)0xFF)
			|| (cf_e2prom_emu_instance.current_record != (uint16_t)0)){
		/* The record number increments. */
		uint16_t tmp_current_record = (uint16_t)(cf_e2prom_emu_instance.current_record + 1);
		/* Checks whether the record is maximum in the page. */
		if(tmp_current_record >= CF_E2ROM_EMU_NUM_OF_RECORD){
			uint32_t record_d;
			fc_code_flash_page_number_t tmp_page = (fc_code_flash_page_number_t)(cf_e2prom_emu_instance.current_page+1);
			if(tmp_page > cf_e2prom_emu_instance.last_page){
				tmp_page = cf_e2prom_emu_instance.first_page;
			}
			record_d = cf_e2prom_emu_read_record_d(tmp_page);
			if((record_d < CF_E2ROM_EMU_RECORD_D_REWRITE_MAX) || (record_d == (uint32_t)0xFFFFFFFFUL)){
				/* Erases it and checks brank it. */
				if(record_d != (uint32_t)0xFFFFFFFFUL){
					result = fc_erase_page_code_flash(tmp_page, (uint8_t)1);
					if(result != TXZ_SUCCESS){
						/* Erasing failure */
						result = TXZ_ERROR;
					}						
				}
				if(result == TXZ_SUCCESS){
					/* Prepares writing Record-D. */
					uint8_t src[FC_CODE_FLASH_WRITE_SIZE] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
															 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00};
					uint8_t* dst_address = (uint8_t*)(FC_CODE_FLASH_ADDRESS_TOP
														+ ((uint32_t)tmp_page * (uint32_t)FC_PAGE_SIZE)
														+ CF_E2ROM_EMU_RECORD_D_WRITE_POSITION);
					record_d++;
					*(uint32_t*)&src[0x0C] = CF_E2ROM_EMU_RECORD_D_REWRITE_MAX;
					/* Writes and verifies the Record-D. */
					result = fc_write_code_flash(src, dst_address, (uint32_t)FC_CODE_FLASH_WRITE_SIZE);
					if(result == TXZ_SUCCESS){
						cf_e2prom_emu_instance.current_page = tmp_page;
						cf_e2prom_emu_instance.current_record = 0;
					}
					else{
						/* Write failure of the Record-D. */
						result = TXZ_ERROR;
					}
				}
				else{
					/* Erase failure of the page */
					result = TXZ_ERROR;
				}
			}
			else{
				/* It used up the record area. */
				result = TXZ_ERROR;
			}
		}
		else{
			/* The record number increments. */
			cf_e2prom_emu_instance.current_record++;
		}
	}

	if(result == TXZ_SUCCESS){
		/* Writes and verifies the record. */
		uint8_t* dst_address = (uint8_t*)(FC_CODE_FLASH_ADDRESS_TOP
											+ ((uint32_t)cf_e2prom_emu_instance.current_page * (uint32_t)FC_PAGE_SIZE)
											+ (cf_e2prom_emu_instance.current_record * cf_e2prom_emu_instance.recod_size));
		result = fc_write_code_flash(cf_e2prom_emu_instance.data_address, dst_address, (uint32_t)cf_e2prom_emu_instance.recod_size);
	}

	/* Returns the vector table to internal ROM. */
	/* Lower 6bits of VTOR is a reservation. */ 
#if     defined ( __CC_ARM )    /* ARM Compiler */
	SCB->VTOR = (uint32_t)&Image$$ER_IROM1$$Base;
#elif   defined ( __ICCARM__ )  /* IAR Compiler */
        SCB->VTOR = (uint32_t)((uint32_t *)__section_begin("__ICFEDIT_intvec_start__"));
#endif

	/* Enables the interruption. */
	__set_PRIMASK(CORE_PRIMASK_INT_ENABLE);

	return (result);
}

/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
	uint32_t i,j;
	TXZ_Result retval;

	/* Sets the CG. */
	/* It provides an internal high speed clock to the Flash ROM for the writing and the erasing functions.  */
	cf_e2prom_emu_internal_oscillator_enable();

	/* Initialize. and sends to a RAM area flash control programs. */
	{
		cf_e2prom_emu_init_t init;
		init.first_page = (fc_code_flash_page_number_t)FC_CODE_FLASH_PAGE6;
		init.number_of_pages = (uint8_t)(FC_CODE_FLASH_PAGE8 - FC_CODE_FLASH_PAGE6+1);	/* 3 pages */
		init.data_size = (uint16_t)CF_E2ROM_EMU_DATA_SIZE;
		init.data_address = user_data.user_data;			/*test_user_data*/

		if(cf_e2prom_emu_initialize(&init) != TXZ_SUCCESS){
			for(;;){};
		}
	}

	/* Checks whether flash ROMs are auto operation. */
	while(fc_get_status(FC_SR0_RDYBSY) == TXZ_BUSY){
		;
	}

	/* Blank Check */
	/* When the E2PROM emulation area is the blank, it formats that area. */
	/* But if that area is already used, it restores a state. */
	retval = cf_e2prom_emu_blank_check_using_pages();
	if(retval == TXZ_SUCCESS){
		/* Formats an E2PROM area in the code flash ROM. */
		retval = cf_e2prom_emu_format();
		if(retval != TXZ_SUCCESS){
			/* It cannot use. */
			for(;;){};
		}

		/* Sets user data. */
		(void)memcpy(&user_data.user_data, test_user_data, CF_E2ROM_EMU_DATA_SIZE);
	}
	else{
		/* Restore a state before a reset. */
		retval = cf_e2prom_emu_restore();
		if(retval != TXZ_SUCCESS){
			/* It cannot use. */
			for(;;){};
		}
		if(cf_e2prom_emu_read_written_mark_current_record() == CF_E2ROM_EMU_WRITTEN_MARK){
			/* Reads user data. */
			cf_e2prom_emu_read_data();
		}
		else{
			/* Sets user data. */
			(void)memcpy(&user_data.user_data, test_user_data, CF_E2ROM_EMU_DATA_SIZE);
		}
	}

	/****************************************/
	/*	Test of writing to E2PROM area		*/
	/****************************************/
	/* Writes records on three pages(j). */
	for(j=0; j<3; j++){
		/* Writes record for one page. */
		for(i=0; i<CF_E2ROM_EMU_NUM_OF_RECORD; i++){
			/* Writes one record. */
			retval = cf_e2prom_emu_write_record();
			/* An error returns, */
			/* if it fails to writing, to erasing and formatting a new pape. */
			if(retval == TXZ_ERROR){
				for(;;){};
			}

		/* Sets the next user data. */
		user_data.user_data[0]++;	/* Increments test user data */
		}
	}

	/* Writes one record at first page of the record 0 once more */
	retval = cf_e2prom_emu_write_record();
	/* An error returns, */
	/* if it fails to writing, to erasing and formatting a new pape. */
	if(retval == TXZ_ERROR){
		for(;;){};
	}

	/* Finish. */
	for(;;){};

}

/**
 *  @} 
 */ /* End of group CODE_FLASH_E2PROM_EMULATION */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
