/** 
 *******************************************************************************
 * @file    main.c
 * @brief   GPIO LED Sample Application.
 * @version V1.0.1.3
 * $Date:: 2017-11-09 10:38:27 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup GPIO_LED
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_sw.h"
#include "bsp_led.h"
/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    sw_t        sw[(uint32_t)BSP_PSW_MAX];  /*!< Application :Push SW            */
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    gpio_t      gpio_obj;            /*!< Application :GPIO. */
} instance_t;


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
static instance_t instance; /*!< Instance.                    */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
#define MAIN_NULL       ((void *)0)     /*!< NULL. */
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            p_led->init.state           = LED_STATE_OFF;
            led_initialize(p_led);
        }
    }
    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    {
        uint32_t i;
        sw_t *p_sw = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            p_sw = &instance.sw[i];

            p_sw->init.id                   = (uint32_t)p_sw;
            p_sw->init.p_gpio               = bsp_get_gpio_instance_address();
            p_sw->init.port.group           = bsp_get_gpio_group_psw((BSPPsw)(i));
            p_sw->init.port.num             = bsp_get_gpio_num_psw((BSPPsw)(i));
            p_sw->init.polling.func         = TXZ_DISABLE;
            p_sw->init.polling.interval     = 0;
            p_sw->init.chattering.func      = TXZ_DISABLE;
            p_sw->init.chattering.count     = 0;
            p_sw->init.handler              = MAIN_NULL;
            sw_initialize(p_sw);
        }
    }

    return (result);
}

/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
void irq_systick(void);
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
      uint32_t period;    // Period for SYSTICK
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    /* Period for 10msec SYSTICK */
    SystemCoreClockUpdate();
    period = SystemCoreClock / 100;
     
    if (SysTick_Config(period ))    /* systick interrupt cycle setting */
    {
    }
    NVIC_EnableIRQ(SysTick_IRQn);   /* systick interrupt enable */

    /* dummy loop */
    while (1){
        __NOP();
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Systick IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_systick(void)
{
    uint32_t i;
    led_t *p_led = MAIN_NULL;
    gpio_pinstate_t val = GPIO_PIN_SET;


    /* Check SW Push(= GPIO_PIN_RESET) */
        for (i=0; i < (uint32_t)BSP_PSW_MAX; i++)
        {

            if(gpio_read_bit(&instance.gpio_obj, bsp_get_gpio_group_psw((BSPPsw)i), bsp_get_gpio_num_psw((BSPPsw)i), GPIO_Mode_DATA, &val) != TXZ_ERROR)
            {
            }
            
            if(val == GPIO_PIN_RESET){
                p_led = &instance.led[i];
                led_turn_on(p_led);
            }else{
                p_led = &instance.led[i];
                led_turn_off(p_led);
            }
        }
    }

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __nop();
    }
}
#endif
/**
 *  @} 
 */ /* End of group GPIO_LED */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
