/**
 *******************************************************************************
 * @file    lvd.c
 * @brief   This file provides API functions for LVD.
 * @version V1.0.0.7
 * $Date:: 2018-07-12 11:03:45 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "txz_driver_def.h"
#include "lvd.h"

#if defined(__LVD_H)

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup LVD
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group LVD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */
#define LVD_UPPER           ((uint32_t)0x00000000)          /*!< LVD UPPER                */
#define LVD_LOWER           ((uint32_t)0x00000001)          /*!< LVD LOWER                */
#if defined(TMPM4K4) || defined(TMPM4K4A)
#define LVD_VOLTAGE_26      ((uint32_t)0x00000000)          /*!< LVD voltage 2.6+-0.1V    */
#define LVD_VOLTAGE_27      ((uint32_t)0x00000010)          /*!< LVD voltage 2.7+-0.1V    */
#define LVD_VOLTAGE_28      ((uint32_t)0x00000020)          /*!< LVD voltage 2.8+-0.1V    */
#define LVD_VOLTAGE_29      ((uint32_t)0x00000030)          /*!< LVD voltage 2.9+-0.1V    */
#define LVD_VOLTAGE_40      ((uint32_t)0x00000040)          /*!< LVD voltage 4.0+-0.1V    */
#define LVD_VOLTAGE_42      ((uint32_t)0x00000050)          /*!< LVD voltage 4.2+-0.1V    */
#define LVD_VOLTAGE_44      ((uint32_t)0x00000060)          /*!< LVD voltage 4.4+-0.1V    */
#define LVD_VOLTAGE_46      ((uint32_t)0x00000070)          /*!< LVD voltage 4.6+-0.1V    */
#elif defined(TMPM3H6)
#define LVD_VOLTAGE_26      ((uint32_t)0x00000000)          /*!< LVD voltage 2.6+-0.1V    */
#define LVD_VOLTAGE_27      ((uint32_t)0x00000010)          /*!< LVD voltage 2.7+-0.1V    */
#define LVD_VOLTAGE_28      ((uint32_t)0x00000020)          /*!< LVD voltage 2.8+-0.1V    */
#define LVD_VOLTAGE_29      ((uint32_t)0x00000030)          /*!< LVD voltage 2.9+-0.1V    */
#define LVD_VOLTAGE_38      ((uint32_t)0x00000040)          /*!< LVD voltage 3.8+-0.1V    */
#define LVD_VOLTAGE_40      ((uint32_t)0x00000050)          /*!< LVD voltage 4.0+-0.1V    */
#define LVD_VOLTAGE_42      ((uint32_t)0x00000060)          /*!< LVD voltage 4.2+-0.1V    */
#define LVD_VOLTAGE_44      ((uint32_t)0x00000070)          /*!< LVD voltage 4.4+-0.1V    */
#elif defined(TMPM3HQ)
#define LVD_VOLTAGE_26      ((uint32_t)0x00000000)          /*!< LVD voltage 2.6+-0.1V    */
#define LVD_VOLTAGE_27      ((uint32_t)0x00000010)          /*!< LVD voltage 2.7+-0.1V    */
#define LVD_VOLTAGE_28      ((uint32_t)0x00000020)          /*!< LVD voltage 2.8+-0.1V    */
#define LVD_VOLTAGE_29      ((uint32_t)0x00000030)          /*!< LVD voltage 2.9+-0.1V    */
#define LVD_VOLTAGE_38      ((uint32_t)0x00000040)          /*!< LVD voltage 3.8+-0.1V    */
#define LVD_VOLTAGE_40      ((uint32_t)0x00000050)          /*!< LVD voltage 4.0+-0.1V    */
#define LVD_VOLTAGE_42      ((uint32_t)0x00000060)          /*!< LVD voltage 4.2+-0.1V    */
#define LVD_VOLTAGE_44      ((uint32_t)0x00000070)          /*!< LVD voltage 4.4+-0.1V    */
#endif
#define LVD_SEL_INT         ((uint32_t)0x00000004)          /*!< LVD INTLVD               */
#define LVD_SEL_RESET       ((uint32_t)0x00000000)          /*!< LVD reset                */
#define LVD_OUT_ENABLE      ((uint32_t)0x00000002)          /*!< LVD output enable        */
#define LVD_OUT_DISABLE     ((uint32_t)0xFFFFFFFD)          /*!< LVD output disable       */
#define LVD_ENABLE          ((uint32_t)0x00000001)          /*!< LVD enable               */
#define LVD_DISABLE         ((uint32_t)0xFFFFFFFE)          /*!< LVD disable              */
/** 
 *  @}
 */ /* End of group LVD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group LVD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_typedef LVD Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group LVD_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_fuctions LVD Private Fuctions
 *  @{
 */


/**
 *  @}
 */ /* End of group LVD_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup LVD_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the LVD object.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void init_LVD(void)
{
    TSB_LVD->CR &= (uint8_t)LVD_DISABLE;                 /* LVD Disable */
    TSB_LVD->CR |= (uint8_t)LVD_VOLTAGE_40;              /* 4.0V */
    TSB_LVD->CR |= (uint8_t)LVD_SEL_RESET;               /* RESET */
    TSB_LVD->CR &= (uint8_t)LVD_OUT_DISABLE;             /* Output Disable */
}

/*--------------------------------------------------*/
/**
  * @brief     Start the LVD object.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void start_LVD(void)
{
    TSB_LVD->CR |= (uint8_t)LVD_ENABLE;                  /* LVD Enable */
}

/**
 *  @} 
 */ /* End of group LVD_Exported_functions */
 
/**
 *  @} 
 */ /* End of group LVD */
 
/**
 *  @} 
 */ /* End of group Periph_Driver */
 
#endif /* defined(__LVD_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

