/**
 *******************************************************************************
 * @file    system_TRMOSC.c
 * @brief   TRMOSC Sample Application.
 * @version 1.0.0.0
 * $Date:: 2017-12-21 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include "txz_sample_def.h"

/*-------- <<< Start of configuration section >>> ----------------------------*/

/* Semi-Independent Watchdog Timer (SIWDT) Configuration */
#define SIWDEN_Val              (0x00000000UL)            /* SIWD Disable */
#define SIWDCR_Val              (0x000000B1UL)            /* SIWD Disable code */

/*-------- <<< End of configuration section >>> ------------------------------*/

/*-------- DEFINES -----------------------------------------------------------*/
/* Define clocks */
#define IOSC_10M                (10000000UL)
#define IXTALH                  IOSC_10M        /* Internal high-speed oscillator freq */

/* Determine core clock frequency according to settings */
/* System clock is high-speed clock*/
#define CORE_TALH (IXTALH)
#define __CORE_CLK   (CORE_TALH)
#define __CORE_SYS   (__CORE_CLK)


/* Clock Variable definitions */
uint32_t SystemCoreClock = __CORE_SYS;  /*!< System Clock Frequency (Core Clock) */

/**
 * Initialize the system
 *
 * @param  -
 * @return -
 *
 * @brief  Update SystemCoreClock according register values.
 */
void SystemCoreClockUpdate(void)
{                               /* Get Core Clock Frequency */
    SystemCoreClock = IXTALH;
}

/**
 * Initialize the system
 *
 * @param  -
 * @return -
 *
 * @brief  Setup the microcontroller system.
 *         Initialize the System.
 */
void SystemInit(void)
{
    /* SIWD Disable */
    TSB_SIWD0->EN = SIWDEN_Val;
    TSB_SIWD0->CR = SIWDCR_Val;

    /* Internal HOSC Enable (Setting after a Reset) */
#if (__FPU_PRESENT == 1) && (__FPU_USED == 1)   /* FPU setting */
    SCB->CPACR |= ((3UL << 10*2)|(3UL << 11*2));  /* set CP10 and CP11 Full Access */
#endif
}
